#include "Debug.h"

#include <kdebug.h>

void PrintNumber(u32 value)
{
    KDebug_AlertNumber(value);
}

void Print(char* message)
{
    KDebug_Alert(message);
}

void PrintSprite(const SpriteDef* sprite)
{
    PrintNumber(sprite->posx);
    PrintNumber(sprite->posy);
    PrintNumber(sprite->tile_attr);
    PrintNumber(sprite->size);
    PrintNumber(sprite->link);
}

void PrintType(u16 objectType)
{
    switch (objectType)
    {
    case TYPE_PLAYER_PROJECTILE: Print("Projectile");break;
    case TYPE_EFFECT: Print("Effect");break;
    case TYPE_ENEMY: Print("Enemy");break;
    case TYPE_ENEMY_PROJECTILE: Print("EnemyProjectile");break;
    case TYPE_PICKUP: Print("Pickup");break;
    case TYPE_BACKGROUNDOBJECT: Print("BackgroundObj");break;
    }
}

void PrintObject(ObjectStruct* object)
{
    PrintNumber((u32)object);
    PrintType(object->objectType);
    PrintNumber((u32)object->nextObjectOfSameType);
    if (object->nextObjectOfSameType)
        PrintType(object->nextObjectOfSameType->objectType);
    PrintNumber((u32)object->previousObjectOfSameType);
    if (object->previousObjectOfSameType)
        PrintType(object->previousObjectOfSameType->objectType);
    PrintNumber((u32)object->animation);
}

void PrintObjectStruct(ObjectStruct* object)
{
    Print("****** OBJECT *******");
    PrintType(object->objectType);
    PrintNumber((u32)object);
    PrintNumber((u32)object->nextObjectOfSameType);
    if (object->nextObjectOfSameType)
        PrintType(object->nextObjectOfSameType->objectType);
    PrintNumber((u32)object->previousObjectOfSameType);
    if (object->previousObjectOfSameType)
        PrintType(object->previousObjectOfSameType->objectType);
    Print("position, screen pos, speedxy");
    PrintNumber(object->x);
    PrintNumber(object->y);
    PrintNumber(object->screenx);
    PrintNumber(object->screeny);
    PrintNumber(object->speedx);
    PrintNumber(object->speedy);
    Print("tile attribute, update, draw, collision functions");
    PrintNumber(object->tileAttribute);
    PrintNumber((u32)object->updateFunction);
    PrintNumber((u32)object->drawFunction);
    PrintNumber((u32)object->collisionFunction);
    Print("lifetime, health, damage");
    PrintNumber(object->lifetime);
    PrintNumber(object->healthPoints);
    PrintNumber(object->damage);
    Print("rect left, right, top, bottom");
    PrintNumber(object->rectLeft);
    PrintNumber(object->rectRight);
    PrintNumber(object->rectTop);
    PrintNumber(object->rectBottom);

    Print("*********************");
}

void PrintAPLAN(u32 value, u16 x, u16 y)
{
    char tempString[16];
    intToStr(value, tempString, 16);
    VDP_drawText(tempString, x, y);
}

void ErrorMessage(const char* message)
{
    VDP_init();
    VDP_drawText("Error!", 0, 0);
    VDP_drawText(message, 0, 1);

    while(1);
}

void Assert(u16 condition, const char* message)
{
    if (!condition)
    {
        VDP_init();
        VDP_drawText("Assert!", 0, 0);
        VDP_drawText(message, 0, 1);

        while(1);
    }
}
