#include "GGAnimationDraw.h"
#include "Debug.h"
#include "vdp_spr.h"
#include "ObjectStruct.h"
#include "ObjectManager.h"
#include "exported\ExportedGGAnimations.h"
#include "VDP_Extra.h"

#define MAX_SPRITES_TO_DRAW 80
GGSprite* spritesToDraw[MAX_SPRITES_TO_DRAW];
u8 currentSpriteToDraw;

void GGAnimation_Draw(s16 x, 
                      s16 y, 
                      const GGAnimation* animation, 
                      u16 frameNumber, 
                      u16 tileAttribute)
{
    const GGFrame* frame = *(animation->frames + frameNumber);
    const GGSprite* sprite = *(frame->sprites);
    tileAttribute += animation->vdpInfo->startIndex;

    u16 loop = frame->numSprites;

    if (spriteDrawIndex + frame->numSprites > MAX_SPRITE)
    {
        loop = MAX_SPRITE - spriteDrawIndex;
    }

    for (; loop != 0; loop--)
    {
        if (GET_DIRECTION(tileAttribute)== DIRECTION_RIGHT)
        {
            currentSpriteDef->posx = x + sprite->xoffset;
        }
        else
        {
            currentSpriteDef->posx = x + sprite->xflippedoffset;
        }

        currentSpriteDef->posy = y + sprite->yoffset;
        currentSpriteDef->tile_attr = tileAttribute + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;

        currentSpriteDef++;
        sprite++;
    }
}

void GGAnimation_DrawNoFlip(s16 x, 
                            s16 y, 
                            const GGAnimation* animation, 
                            u16 frameNumber, 
                            u16 tileAttribute)
{
    const GGFrame* frame = *(animation->frames + frameNumber);
    const GGSprite* sprite = *(frame->sprites);
    tileAttribute += animation->vdpInfo->startIndex;

    u16 loop = frame->numSprites;

    if (spriteDrawIndex + frame->numSprites > MAX_SPRITE)
    {
        loop = MAX_SPRITE - spriteDrawIndex;
    }

    for (; loop != 0; loop--)
    {
        currentSpriteDef->posx = x + sprite->xoffset;
        currentSpriteDef->posy = y + sprite->yoffset;
        currentSpriteDef->tile_attr = tileAttribute + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;

        currentSpriteDef++;
        sprite++;
    }
}


void GGAnimation_DrawObject(ObjectStruct* object)
{
    const GGSprite* sprite = *(object->currentAnimationFrame->sprites);
    u16 loop = object->currentAnimationFrame->numSprites;

    if (spriteDrawIndex + object->currentAnimationFrame->numSprites > MAX_SPRITE)
    {
        loop = MAX_SPRITE - spriteDrawIndex;
    }

    for (; loop != 0; loop--)
    {
        if (GET_DIRECTION(object->tileAttribute) == DIRECTION_RIGHT)
        {
            currentSpriteDef->posx = object->screenx + sprite->xoffset;
        }
        else
        {
            currentSpriteDef->posx = object->screenx + sprite->xflippedoffset;
        }

        currentSpriteDef->posy = object->screeny + sprite->yoffset;
        currentSpriteDef->tile_attr = object->tileAttribute + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;

        currentSpriteDef++;
        sprite++;
    }
}

void GGAnimation_DrawObjectNoFlip(ObjectStruct* object)
{
    const GGSprite* sprite = *(object->currentAnimationFrame->sprites);
    u16 loop = object->currentAnimationFrame->numSprites;

    if (spriteDrawIndex + object->currentAnimationFrame->numSprites > MAX_SPRITE)
    {
        loop = MAX_SPRITE - spriteDrawIndex;
    }

    while (loop--)
    {
        currentSpriteDef->posx = object->screenx + sprite->xoffset;
        currentSpriteDef->posy = object->screeny + sprite->yoffset;
        currentSpriteDef->tile_attr = object->tileAttribute + sprite->VDPTileOffset;
        currentSpriteDef->size = sprite->sgdkSpriteSize;
        spriteDrawIndex++;
        currentSpriteDef->link = spriteDrawIndex;

        currentSpriteDef++;
        sprite++;
    }
}

