#include "HUDManager.h"

#include <genesis.h>
#include <KDebug.h>
#include <vdp_spr.h>
#include <vdp_tile.h>

#include "Collisions.h"
#include "Debug.h"
#include "generated\DebugSettings.h"
#include "GameFont.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "PlayerManager.h"
#include "ObjectManager.h"
#include "exported\ExportedGenResSprites.h"
#include "exported\ExportedGGAnimations.h"
#include "VDP_Extra.h"
#include "VDPManager.h"
#include "WeaponManager.h"


#define SHOW_LIVES_TIME 160
u16 showLivesTimer = 0;

u32 oldFPS;
u16 fpsCharacter1;
u16 fpsCharacter2;
u16 fpsCharacter3;

u16 oldStars;
u16 starsCharacter1;
u16 starsCharacter2;
u16 starsCharacter3;

u16 wandAttribute;

u16 penguinHeadAttr;

inline void UpdateFPSString()
{
}

u16 livesCharacter1;
u16 livesCharacter2;

u16 characterAttr;
u16 starsNumberAttr;

void HUDManager_Update()
{
    // update lives string.
    if (showLivesTimer > 0)
    {
        u16 lives = playerData.lives;
        livesCharacter2 = (lives % 10) + characterAttr;
        lives /= 10;
        livesCharacter1 = (lives % 10) + characterAttr;
    }

    // update fps
    u16 fps = getFPS();

    if (fps != oldFPS)
    {
        u16 n3 = fps % 10;
        fps /= 10;
        u16 n2 = fps % 10;
        fps /= 10;
        u16 n1 = fps % 10;

        fpsCharacter1 = characterAttr + n1;
        fpsCharacter2 = characterAttr + n2;
        fpsCharacter3 = characterAttr + n3;

        oldFPS = fps;
    }

    if (numStars[currentWeapon] != oldStars)
    {
        u16 n1;
        u16 n2;

        if (numStars[currentWeapon] != 100)
        {
            u16 temp = numStars[currentWeapon];
            n2 = temp % 10;
            temp /= 10;
            n1 = temp % 10;
        }
        else
        {
            n1 = 11;
            n2 = 12;
        }


        starsCharacter1 = starsNumberAttr + (n1 << 2);
        starsCharacter2 = starsNumberAttr + (n2 << 2);

        oldStars = numStars[currentWeapon];
    }

    UpdateFPSString();
}

void HUDManager_Init()
{
    oldFPS = 0;
    oldStars = 101;

    penguinHeadAttr = TILE_ATTR_FULL(PAL0, 1, 0, 0, penguinHeadVDPInfo.startIndex);
    characterAttr = TILE_ATTR_FULL(PAL0, 1, 0, 0, fontTileIndex + 16);
    starsNumberAttr = TILE_ATTR_FULL(PAL1, 1, 0, 0, numbersVDPInfo.startIndex);
    starsCharacter3 = starsNumberAttr + (10 << 2); // percentage sign.

    wandAttribute = TILE_ATTR_FULL(PAL1, 1, 0, 0, wandIconVDPInfo.startIndex);
}

void HUDManager_UpdateLives()
{
    showLivesTimer = SHOW_LIVES_TIME;
}

//s16 hoverTable[16] = {16,22,27,31,32,31,27,22,16,10,5,1,0,1,5,10};

s16 hoverTable[16] = {2,3,3,4,4,4,3,3,2,1,1,0,0,0,1,1};

void HUDManager_Draw()
{
    // update lives string.
    if (showLivesTimer > 0)
    {
        s16 x = playerObject->screenx;
        s16 y = playerObject->screeny + 28;

        MyVDP_setSprite(x - 8, y, SPRITE_SIZE(2,2), penguinHeadAttr);

        y += 4;
        MyVDP_setSprite(x + 8, y, SPRITE_SIZE(1,1), characterAttr + 'x' - 48);
        MyVDP_setSprite(x + 16, y, SPRITE_SIZE(1,1), livesCharacter1);
        MyVDP_setSprite(x + 24, y, SPRITE_SIZE(1,1), livesCharacter2);

        showLivesTimer--;
    }

    if (debugSettings.showFPSCounter)
    {
        MyVDP_setSprite(144, 8, SPRITE_SIZE(1,1), fpsCharacter1);
        MyVDP_setSprite(152, 8, SPRITE_SIZE(1,1), fpsCharacter2);
        MyVDP_setSprite(160, 8, SPRITE_SIZE(1,1), fpsCharacter3);
    }

    MyVDP_setSprite(153, 200, SPRITE_SIZE(2,2), starsCharacter1);
    MyVDP_setSprite(167, 200, SPRITE_SIZE(2,2), starsCharacter2);
    MyVDP_setSprite(181, 200, SPRITE_SIZE(2,2), starsCharacter3);
    MyVDP_setSprite(123, 196, SPRITE_SIZE(3,3), wandAttribute);
}


