#include "LevelManager.h"

#include "Collisions.h"
#include "Config.h"
#include "Debug.h"
#include "DebugMenu.h"
#include "GameFont.h"
#include "generated\DebugSettings.h"

#include "generated\AllObjects.h"
#include "generated\AllLevels.h"

#include "exported\ExportedGenResSprites.h"
#include "exported\ExportedGGAnimations.h"
#include "FadeManager.h"
#include "HUDManager.h"
#include "JoypadManager.h"
#include "MathUtils.h"
#include "ObjectManager.h"
#include "PlayerManager.h"
#include "ScrollManager.h"
#include "StateManager.h"
#include "VDP_Extra.h"
#include "DrawManager.h"
#include "WeaponManager.h"

/*********************************************************
*********************************************************/
#define FIRST_LEVEL level001
/*********************************************************
*********************************************************/

const Level* currentLevel;
const VPMap* currentForegroundMap = NULL;
const VPMap* currentBackgroundMap = NULL;


void LevelManager_SetLevel(const Level* level)
{
    currentLevel = level;
    currentForegroundMap = currentLevel->foregroundMap;
    currentBackgroundMap = currentLevel->backgroundMap;
}

void LevelManager_Init()
{
    LevelManager_SetLevel(&FIRST_LEVEL);

    PlayerManager_SetupForGame();

    MyVDP_waitVSync();

    VDP_setEnable(0);

    VDP_setEnable(1);

    StateManager_ChangeToState(LevelManager_StartLevel);
}

void LevelManager_LoadCommonTilesToVDP()
{
    VDPManager_ResetVDPInfo();

    LOAD_GENRES_SPRITE_TO_VDP(bullet);
    //LOAD_GENRES_SPRITE_TO_VDP(heart);
    LOAD_GENRES_SPRITE_TO_VDP(penguinHead);
    LOAD_GENRES_SPRITE_TO_VDP(numbers);
    LOAD_GENRES_SPRITE_TO_VDP(wandIcon);

    VDPManager_loadFont(gameFont, FONT_LEN);

    //VDPManager_ReserveVDPArea(&playerVDPInfo, 100);
}

void LevelManager_StartLevel()
{
    MyVDP_waitVSync();

    FadeManager_SetAllBlack();

    LevelManager_LoadCommonTilesToVDP();

    HUDManager_Init();

    ObjectManager_Init();

    currentLevel->InitLevel();

    PlayerManager_SetupLife();
    ScrollManager_Init();
    ObjectStreamer_Init();

    StateManager_ChangeToState(LevelManager_UpdateLevel);
}



void LevelManager_UpdateLevel()
{
    currentLevel->UpdateLevel();
    ScrollManager_Update();

    ObjectStreamer_Update();
    ObjectManager_UpdateObjects();

    HUDManager_Update();
    DrawManager_Draw();

    if (joystickState.startPressed)
    {
        StateManager_ChangeToState(DebugMenu_Start);
    }

    // wait for vsync and then draw
    if (debugSettings.waitforvsync != FALSE)
    {
        MyVDP_waitVSync();
    }

    VDP_updateSprites();
    ScrollManager_UpdateVDP();
    WeaponManager_UpdateVDP();
}

void LevelManager_ReturnFromMenu()
{
    MyVDP_waitVSync();

    VDP_setEnable(0);

    ScrollManager_Refresh();

    VDP_setEnable(1);

    StateManager_ChangeToState(LevelManager_UpdateLevel);
}

void LevelManager_SwitchToLevel(u16 levelNumber)
{
    LevelManager_SetLevel(levels[levelNumber]);

    StateManager_ChangeToState(LevelManager_StartLevel);
}

void LevelManager_RestartLevel()
{
    StateManager_ChangeToState(LevelManager_StartLevel);
}

