#ifndef OBJECTSTRUCT_INCLUDE_H
#define OBJECTSTRUCT_INCLUDE_H

#include <genesis.h>
#include <vdp_spr.h>

#include "GGAnimationTypes.h"

// object types.
#define TYPE_PLAYER_BULLET      0
#define TYPE_PLAYER_PROJECTILE  1
#define TYPE_EFFECT             2
#define TYPE_ENEMY              3
#define TYPE_ENEMY_PROJECTILE   4
#define TYPE_PICKUP             5
#define TYPE_BACKGROUNDOBJECT   6
#define TYPE_BOSS               7
#define TYPE_GRABBER            8
#define MAX_OBJECT_TYPES        9
// specially treated object types.
#define TYPE_PLAYER             10


typedef struct object_struct
{
    struct object_struct* previousObjectOfSameType;
    struct object_struct* nextObjectOfSameType;
    s16 x;
    s16 y;
    s16 screenx;
    s16 screeny;
    u16 tileAttribute;
    void (*updateFunction)(struct object_struct* objectStruct);
    void (*drawFunction)(struct object_struct* objectStruct);
    void (*collisionFunction)(struct object_struct* objectStruct, struct object_struct* collidedObject);
    void (*grabbedFunction)(struct object_struct* objectStruct, struct object_struct* collidedObject);
    const GGAnimation* animation;
    s16 speedx;
    s16 speedy;
    u16 lifetime; // one tick every frame. 60 ticks every second. 3600 ticks every minute. Max 18.2 minutes. only for temporary objects.
    u16 objectType;
    s16 rectLeft;
    s16 rectRight;
    s16 rectTop;
    s16 rectBottom;
    s16 healthPoints;
    u16 damage;
    u16 animationFrameCounter;
    const GGFrame* currentAnimationFrame;
    u16 damagedTimer;
    s32 extra1;
    s32 extra2;
    s32 extra3;
    s32 extra4;
} object_struct;

typedef struct object_struct ObjectStruct;

typedef ObjectStruct* (*ObjectCreateFunctionType) (s16 x, s16 y, u32* properties, u16 numProperties);
typedef void (*ObjectFunctionType)(ObjectStruct* objectStruct);
typedef void (*ObjectCollisionFunctionType)(ObjectStruct* objectStruct, ObjectStruct* collidedObject);

#endif
