#include "ScrollManager.h"

#include "Config.h"
#include "Debug.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "VDPPlaneManager.h"
#include "VDP_Extra.h"
#include "PlayerManager.h"


ScrollDataStruct foregroundScrollData;
ScrollDataStruct backgroundScrollData;

u16 backgroundScrollValues[28];

u16 scrollLayer1 = -1;
u16 scrollLayer2 = -1;
u16 scrollLayer3 = -1;
u16 scrollLayer4 = -1;

s16 scrollLayer1Speed = 0;
s16 scrollLayer2Speed = 0;
s16 scrollLayerSpeed = 0;
s16 scrollLayer4Speed = 0;

void ScrollManager_UpdateScrollData();
void ScrollManager_SetupMaps();


void ScrollManager_Init()
{
    scrollLayer1 = -1;
    scrollLayer2 = -1;
    scrollLayer3 = -1;
    scrollLayer4 = -1;

    scrollLayer1Speed = 0;
    scrollLayer2Speed = 0;
    scrollLayerSpeed = 0;
    scrollLayer4Speed = 0;

    foregroundScrollData.scrollx_hires_limit = PIXEL_TO_VIRTUAL(TILE_TO_PIXEL(currentForegroundMap->width) - SCREEN_WIDTH);
    foregroundScrollData.scrolly_hires_limit = PIXEL_TO_VIRTUAL(TILE_TO_PIXEL(currentForegroundMap->height) - SCREEN_HEIGHT);

    foregroundScrollData.scrollx_hires_clamped = 0;
    foregroundScrollData.scrolly_hires_clamped = 0;
    foregroundScrollData.scrollx_hires = backgroundScroll;
    foregroundScrollData.scrolly_hires = 0;
    foregroundScrollData.scrollx_vdp = 0;
    foregroundScrollData.scrolly_vdp = 0;
    foregroundScrollData.cameraShakeTimer = 0;

    currentForegroundMap->LoadVDPAssets(currentForegroundMap);
    currentBackgroundMap->LoadVDPAssets(currentBackgroundMap);

    ScrollManager_UpdateScrollData();

    VDPPlaneManager_Init();

    ScrollManager_Refresh();
}

void ScrollManager_Update()
{
    ScrollManager_UpdateScrollData();
    VDPPlaneManager_Update();
}

void ScrollManager_Refresh()
{
    // register 0x0b sets the scroll mode
    // first two bits sets the horizontal scroll mode (whole layer mode (0x00), tile row mode(0x02), or per-line mode(0x03))
    // third bit (0 or 1) sets the vertical scroll mode (whole layer mode or column mode)

    VDP_setReg(0x0b, 0x02);

    VDPPlaneManager_SetupMaps();
    ScrollManager_UpdateVDP();
}

void ScrollManager_UpdateVDP()
{
    VDP_setAllHorizontalScrollTiles(APLAN, -foregroundScrollData.scrollx_vdp);

    MyVDP_setHorizontalScrollTile(BPLAN, 0, backgroundScrollValues, 28, 0);

    VDP_setVerticalScroll(PLAN_A, foregroundScrollData.scrolly_vdp);
    VDP_setVerticalScroll(PLAN_B, backgroundScrollData.scrolly_vdp);

    VDPPlaneManager_UpdateVDP();
}

void UpdateBackgroundLayers()
{
    if (scrollLayer1 != (u16)(-backgroundScrollData.scrollx_vdp >> 3))
    {
        scrollLayer1 = (u16)(-backgroundScrollData.scrollx_vdp >> 3);

        memsetU16(backgroundScrollValues, scrollLayer1, 6);
    }

    if (scrollLayer2 != (u16)(-backgroundScrollData.scrollx_vdp >> 2))
    {
        scrollLayer2 = (u16)(-backgroundScrollData.scrollx_vdp >> 2);

        memsetU16(backgroundScrollValues + 6, scrollLayer2, 9);
    }

    if (scrollLayer3 != (u16)(-backgroundScrollData.scrollx_vdp >> 1))
    {
        scrollLayer3 = (u16)(-backgroundScrollData.scrollx_vdp >> 1);

        memsetU16(backgroundScrollValues + 15, scrollLayer3, 4);
    }

    scrollLayer4Speed = 0;

    if (scrollLayer4 != (u16)(-backgroundScrollData.scrollx_vdp))
    {
        scrollLayer4Speed = -scrollLayer4;

        scrollLayer4 = (u16)(-backgroundScrollData.scrollx_vdp);

        scrollLayer4Speed += scrollLayer4;

        memsetU16(backgroundScrollValues + 19, scrollLayer4, 9);
    }
}

void ScrollManager_UpdateScrollData()
{
    /*
    foregroundScrollData.scrollx_hires = playerObject->x - TILE_TO_VIRTUAL(TILE_UPDATE_AREA_HALF_WIDTH);
    foregroundScrollData.scrolly_hires = playerObject->y - TILE_TO_VIRTUAL(TILE_UPDATE_AREA_HALF_HEIGHT);
    */

    foregroundScrollData.scrollx_hires_clamped = foregroundScrollData.scrollx_hires;
    foregroundScrollData.scrolly_hires_clamped = foregroundScrollData.scrolly_hires;

    if (foregroundScrollData.scrollx_hires < 0)
    {
        foregroundScrollData.scrollx_hires_clamped = 0;
    }
        
    if (foregroundScrollData.scrollx_hires > foregroundScrollData.scrollx_hires_limit)
    {
        foregroundScrollData.scrollx_hires_clamped = foregroundScrollData.scrollx_hires_limit;
    }        

    if (foregroundScrollData.scrolly_hires < 0)
    {
        foregroundScrollData.scrolly_hires_clamped = 0;
    }
        
    if (foregroundScrollData.scrolly_hires > foregroundScrollData.scrolly_hires_limit)
    {
        foregroundScrollData.scrolly_hires_clamped = foregroundScrollData.scrolly_hires_limit;
    }

    if (foregroundScrollData.cameraShakeTimer > 0)
    {
        foregroundScrollData.scrolly_hires_clamped += (foregroundScrollData.cameraShakeTimer & 1) << 3;
        foregroundScrollData.cameraShakeTimer--;
    }

    foregroundScrollData.scrollx_vdp = VIRTUAL_TO_PIXEL(foregroundScrollData.scrollx_hires_clamped);
    foregroundScrollData.scrolly_vdp = VIRTUAL_TO_PIXEL(foregroundScrollData.scrolly_hires_clamped);

    backgroundScrollData.old_scrollx_vdp = backgroundScrollData.scrollx_vdp;

    backgroundScrollData.scrollx_vdp = foregroundScrollData.scrollx_vdp >> 1;
    backgroundScrollData.scrolly_vdp = foregroundScrollData.scrolly_vdp >> 1;

    backgroundScrollData.scrollx_speed_vdp = backgroundScrollData.old_scrollx_vdp - backgroundScrollData.scrollx_vdp;


    UpdateBackgroundLayers();
}

void ScrollManager_SetCameraShakeTime(u16 time)
{
    if (time > foregroundScrollData.cameraShakeTimer)
    {
        foregroundScrollData.cameraShakeTimer = time;
    }
}
