#include "VDPPlaneManager_VerticalMapFunctions.h"

#include "Config.h"
#include "Debug.h"
#include "LevelManager.h"
#include "MathUtils.h"
#include "VDP_Extra.h"
#include "PlayerManager.h"
#include "ScrollManager.h"

extern u16 vdpHeightMinusOne;

extern DMADataStruct* dmaDataStruct;
extern ScrollDataStruct* scrollData;
extern VPMap* map;
extern u32 plan;
extern u16 palette;
extern u16 vdpPlaneWidth;

static void PrepareMapDMAData(u16 mapRow)
{
    dmaDataStruct->mapDMADestinationAddress = plan + (((mapRow & vdpHeightMinusOne) * vdpPlaneWidth) << 1); 

    u16 tileInfo = TILE_ATTR_FULL(palette, 0, 0, 0, map->tilesetVDPInfo->startIndex);

    // fill the map row dma data
    u16 loop;
    u16* mapDMAData = dmaDataStruct->mapDMAData;
    const u16* mapLocation = map->tilemap + (mapRow * map->width);

    for (loop = map->width; loop > 0; loop--)
    {
        *(mapDMAData) = *(mapLocation) + tileInfo;
        mapDMAData++;
        mapLocation++;
    }
}

void VDPPlaneManager_VerticalMap_Update()
{
    // check current tilePositionY
    s16 tilePositionY = 0;
    dmaDataStruct->updateMap = 0;
    
    if (PIXEL_TO_TILE(scrollData->scrolly_vdp) - 1 > 0)
    {
        tilePositionY = PIXEL_TO_TILE(scrollData->scrolly_vdp) - 1;
    }

    // if it's different from the old one, we'll have to handle updating
    // the plane.
    if (dmaDataStruct->oldTilePosition != tilePositionY)
    {
        u16 newTilePositionY = tilePositionY;

        // here we check the actual position difference of the player
        // since the last frame. Checking the player's y speed is unreliable
        // because it can be set to 0 when falling if hitting solid ground.
        s16 speedy = playerObject->y - playerData.oldPositionY;

        if (speedy > 0)
        {
            // by default the position of the top row is calculated.
            // if the plane is scrolling down, we use the index of the 
            // bottom row for our drawing destination.

            newTilePositionY = (newTilePositionY + TILE_UPDATE_AREA_HEIGHT); 

            if (newTilePositionY > map->height - 1)
            {
                newTilePositionY = map->height - 1;
            }
        }

        // we're assuming we're only updating one row at a time. If the player
        // goes faster than 8 pixels in a frame, we'll miss rows.
        PrepareMapDMAData(newTilePositionY);

        dmaDataStruct->updateMap = 1;
    }

    dmaDataStruct->oldTilePosition = tilePositionY;
}

void VDPPlaneManager_VerticalMap_UpdateVDP()
{
    if (dmaDataStruct->updateMap == 1)
    {
        MyVDP_doDMA(VDP_DMA_VRAM, 
                   (u32)dmaDataStruct->mapDMAData, 
                   dmaDataStruct->mapDMADestinationAddress, 
                   dmaDataStruct->mapDMADataCopyLength, 
                   2);
        VDP_waitDMACompletion();
    }
}

static void DrawRowDMA(u16 mapRow)
{
    u16 arrayStart = mapRow * map->width;
    dmaDataStruct->mapDMADestinationAddress = plan + (((mapRow & vdpHeightMinusOne) * vdpPlaneWidth) << 1); 

    // fill the map row dma data
    u16 loop;
    for (loop = 0; loop < map->width; loop++)
    {
        dmaDataStruct->mapDMAData[loop] = (map->tilemap + arrayStart)[loop] + TILE_ATTR_FULL(palette, 0, 0, 0, map->tilesetVDPInfo->startIndex);
    }

    MyVDP_doDMA(VDP_DMA_VRAM, (u32)dmaDataStruct->mapDMAData, dmaDataStruct->mapDMADestinationAddress, dmaDataStruct->mapDMADataCopyLength, 2);
    VDP_waitDMACompletion();
}

void VDPPlaneManager_VerticalMap_SetupPlane()
{
    s16 tilePositionY = 0;
    
    if (PIXEL_TO_TILE(scrollData->scrolly_vdp) - 1 > 0)
    {
        tilePositionY = PIXEL_TO_TILE(scrollData->scrolly_vdp) - 1;
    }

    dmaDataStruct->oldTilePosition = tilePositionY;

    // compute start and end tiles
    s16 starttiley = tilePositionY;
    s16 endtiley = (tilePositionY + TILE_UPDATE_AREA_HEIGHT); 

    if (endtiley > map->height - 1)
    {
        endtiley = map->height - 1;
    }

    s16 loop = 0;

    for (loop = starttiley; loop <= endtiley; loop++)
    {
        DrawRowDMA(loop);
    }
}

