#include "Boss001.h"

#include "ObjectCommon.h"
#include "..\PlayerManager.h"

void Boss001Update(ObjectStruct* object);
void Boss001Intro(ObjectStruct* object);
void Boss001Exit(ObjectStruct* object);

void EmptyDraw(ObjectStruct* object) {}
void EmptyCollision(ObjectStruct* object, ObjectStruct* collidedObject) {}

ObjectStruct* Boss001Create(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_BOSS,
                                                      Boss001Intro, 
                                                      EmptyDraw,
                                                      EmptyCollision,
                                                      NULL);

    object->x = x;
    object->y = y;

    return object;
}

void Boss001Intro(ObjectStruct* object)
{
    if (object->lifetime == 120)
    {
        EffectCreate(&PlayerHit, object->x - T2V(7), object->y + T2V(2), 0);
        EffectCreate(&PlayerHit, object->x - T2V(15), object->y + T2V(10), 0);
        EffectCreate(&PlayerHit, object->x - T2V(7), object->y + T2V(18), 0);

        EffectCreate(&PlayerHit, object->x - T2V(18), object->y + T2V(22), 0);
        EffectCreate(&PlayerHit, object->x - T2V(28), object->y + T2V(22), 0);
    }

    if (object->lifetime == 125)
    {
        object->extra1 = (u32)HomeBoyCreate(object->x - T2V(8), object->y, NULL, 0);
        object->extra2 = (u32)HomeBoyCreate(object->x - T2V(16), object->y + T2V(8), NULL, 0);
        object->extra3 = (u32)HomeBoyCreate(object->x - T2V(8), object->y + T2V(16), NULL, 0);

        MiniHomeBoyCreate(object->x - T2V(18), object->y + T2V(22), NULL, 0);
        MiniHomeBoyCreate(object->x - T2V(28), object->y + T2V(22), NULL, 0);
    }

    if (object->lifetime > 125)
    {
        ((ObjectStruct*)object->extra1)->lifetime = 0;
        ((ObjectStruct*)object->extra2)->lifetime = 0;
        ((ObjectStruct*)object->extra3)->lifetime = 0;
    }

    if (object->lifetime > 250)
    {
        ((ObjectStruct*)object->extra1)->lifetime = 140;
        ((ObjectStruct*)object->extra2)->lifetime = 280;

        ObjectStruct* childObject;
        
        childObject = (ObjectStruct*)object->extra1;
        SET_RECT_PIXEL_OBJECT(childObject, 8, 34, 12, 47);
        childObject = (ObjectStruct*)object->extra2;
        SET_RECT_PIXEL_OBJECT(childObject, 8, 34, 12, 47);
        childObject = (ObjectStruct*)object->extra3;
        SET_RECT_PIXEL_OBJECT(childObject, 8, 34, 12, 47);

        /*S
        ObjectManager_SwitchType((ObjectStruct*)object->extra1, TYPE_ENEMY);
        ObjectManager_SwitchType((ObjectStruct*)object->extra2, TYPE_ENEMY);
        ObjectManager_SwitchType((ObjectStruct*)object->extra3, TYPE_ENEMY);
        */

        object->updateFunction = Boss001Update;
    }
}

void Boss001Update(ObjectStruct* object)
{
    if (object->extra1)
    {
        ObjectStruct* homeBoy = (ObjectStruct*)object->extra1;

        if (homeBoy->healthPoints <= 0)
        {
            object->extra1 = 0;
        }
    }

    if (object->extra2)
    {
        ObjectStruct* homeBoy = (ObjectStruct*)object->extra2;

        if (homeBoy->healthPoints <= 0)
        {
            object->extra2 = 0;
        }
    }

    if (object->extra3)
    {
        ObjectStruct* homeBoy = (ObjectStruct*)object->extra3;

        if (homeBoy->healthPoints <= 0)
        {
            object->extra3 = 0;
        }
    }

    if (!object->extra1 &&
        !object->extra2 &&
        !object->extra3)
    {
        object->lifetime = 0;

        ObjectManager_ClearPendingObjects();
//        ObjectManager_EraseAllObjectsOfType(TYPE_ENEMY);
        ObjectManager_EraseAllObjectsOfType(TYPE_ENEMY_PROJECTILE);

        object->updateFunction = Boss001Exit;
    }
}

void Boss001Exit(ObjectStruct* object)
{
    if (object->lifetime > 120)
    {
        backgroundScrollSpeed = 2;
        ObjectManager_DestroyObject(object);
    }
}


