#include "CircleFace.h"

#include "ObjectCommon.h"

void CircleFaceSetup(ObjectStruct* object);
void CircleFaceUpdate(ObjectStruct* object);
void CircleFaceDraw(ObjectStruct* object);
void CircleFaceCollide(ObjectStruct* object, ObjectStruct* collidedObject);

//#define DEBUG_OBJECTS

ObjectStruct* CircleFaceCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      CircleFaceSetup, 
                                                      GGAnimation_DrawObjectNoFlip,
                                                      CircleFaceCollide,
                                                      NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("CircleFace could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = 0;
    object->speedy                  = 0;
    object->damage                  = 1;
    object->healthPoints            = 2;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, CircleFace.vdpInfo->startIndex);
    object->lifetime = 0;

    if (numProperties > 0)
    {
        object->extra1 = (u32)((Path*)properties[0])->pathSegments;
    }
    else
    {
        object->extra1 = 0;
    }

    object->extra2 = 0;

    SET_RECT_PIXEL(3, 17, 3, 17);
    SET_ANIMATION(CircleFace);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("CircleFaceCreate");
#endif

    return object;
}

void CircleFaceSetup(ObjectStruct* object)
{
    PathSegment* pathSegment = (PathSegment*)object->extra1;

    s16 startOffsetX = 0;
    s16 startOffsetY = 0;

    if (pathSegment)
    {
        startOffsetX = pathSegment->speedx * object->extra2;
        startOffsetY = pathSegment->speedy * object->extra2;
    }
    else
    {
        startOffsetX = -3 * object->extra2;
    }

    object->x -= startOffsetX;
    object->y -= startOffsetY;

    object->screenx = 300;
    object->screeny = 300;

    object->updateFunction = CircleFaceUpdate;
}

void CircleFaceUpdate(ObjectStruct* object)
{
    if (object->extra1 != 0)
    {
        PathSegment* pathSegment = (PathSegment*)object->extra1;

        object->x += pathSegment->speedx;
        object->y += pathSegment->speedy;

        if (!object->extra2)
        {
            if (pathSegment->time > 0)
            {
                if (object->lifetime > pathSegment->time)
                {
                    object->extra1 = (u32)(pathSegment += 1);
                }
            }
        }
        else
        {
            object->extra2--;

            if (!object->extra2)
            {
                object->lifetime = 0;
            }
        }
    }
    else
    {
        object->x += object->speedx;
    }

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void CircleFaceDrop(ObjectStruct* object)
{
    object->speedy++;
    object->y += object->speedy >> 1;
    object->x += object->speedx;

    UPDATE_SCREEN_POSITION;

    if (!(object->lifetime & 7))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &SmallExplosion);
    }

    if (object->screeny > 224 ||
        object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
    }
    else if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        ObjectManager_DestroyObject(object);
    }
}

void CircleFaceCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->healthPoints == 0)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        StarPickupCreate(object->x - P2V(4), object->y - P2V(4));
        ObjectManager_DestroyObject(object);
    }
}

