#include "CrystalGear.h"

#include "ObjectCommon.h"

void CrystalGearUpdate(ObjectStruct* object);
void CrystalGearDraw(ObjectStruct* object);
void CrystalGearCollide(ObjectStruct* object, ObjectStruct* collidedObject);

ObjectStruct* BlueCrystalGearCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    return CrystalGearCreate(&BlueCrystalGear, x, y, properties, numProperties);
}

ObjectStruct* GreenCrystalGearCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    return CrystalGearCreate(&GreenCrystalGear, x, y, properties, numProperties);
}

ObjectStruct* RedCrystalGearCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    return CrystalGearCreate(&RedCrystalGear, x, y, properties, numProperties);
}

ObjectStruct* BlackCrystalGearCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    return CrystalGearCreate(&BlackCrystalGear, x, y, properties, numProperties);
}


ObjectStruct* CrystalGearCreate(const GGAnimation* animation, s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      CrystalGearUpdate, 
                                                      GGAnimation_DrawObjectNoFlip,
                                                      CrystalGearCollide,
                                                      NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("CrystalGear could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = 0;
    object->speedy                  = 0;
    object->damage                  = 1;
    object->healthPoints            = CRYSTAL_GEAR_HEALTH;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, animation->vdpInfo->startIndex);
    object->lifetime = 0;

    SET_RECT_PIXEL(0, 23, 0, 23);
    SET_ANIMATION_POINTER(animation);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("CrystalGearCreate");
#endif

    return object;
}

void CrystalGearUpdate(ObjectStruct* object)
{
    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void CrystalGearCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (collidedObject->damage < THROW_OBJECT_DAMAGE)
    {
        return;
    }

    ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
    StarPickupCreate(object->x - P2V(4), object->y - P2V(4));
    ObjectManager_DestroyObject(object);
}

