#include "Effect.h"

#include "ObjectCommon.h"

void EffectUpdate(ObjectStruct* object);


void Draw(ObjectStruct* object)
{
    GGAnimation_DrawObject(object);
}

ObjectStruct* EffectCreate(const GGAnimation* animation, s16 x, s16 y, u16 direction)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_EFFECT,
                                                 EffectUpdate, 
                                                 Draw,
                                                 CollisionFunctionStub,
                                                 NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("Effect could not be created!");
#endif
        return 0;
    }

    object->x                           = x;
    object->y                           = y;
    object->tileAttribute               = TILE_ATTR_FULL(PAL0, 1, 0, direction, animation->vdpInfo->startIndex);

    SET_ANIMATION_POINTER(animation);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("EffectCreate");
#endif

    return object;
}

void EffectUpdate(ObjectStruct* object)
{
    UPDATE_ANIMATION_FRAME_NOLOOP_DIE;
    UPDATE_SCREEN_POSITION;
}


void GravityEffectUpdate(ObjectStruct* object)
{
    object->speedy++;
    object->y += object->speedy >> 1;
    object->x += object->speedx;

    UPDATE_ANIMATION_FRAME_LOOP;
    UPDATE_SCREEN_POSITION;

    if (object->screeny > 224)
    {
        ObjectManager_DestroyObject(object);
    }
}

ObjectStruct* StarFallEffect(s16 x, s16 y, s16 speedx, s16 speedy)
{
    ObjectStruct* object = EffectCreate(&StarPickup, x, y, 0);

    object->speedx                      = speedx;
    object->speedy                      = speedy;
    object->tileAttribute               = TILE_ATTR_FULL(PAL1, 1, 0, 0, StarPickup.vdpInfo->startIndex);

    object->updateFunction = GravityEffectUpdate;

    return object;
}

void GrabbedEffectUpdate(ObjectStruct* object)
{
    object->x = playerObject->x + P2V(24);
    object->y = playerObject->y + P2V(3);

    UPDATE_ANIMATION_FRAME_LOOP;
    UPDATE_SCREEN_POSITION;

    object->screeny +=  flyingOffsetY;
}

ObjectStruct* GrabbedEffectCreate()
{
    ObjectStruct* effect = EffectCreate(&GrabbedEffect, 0, 0, 0);

    effect->updateFunction = GrabbedEffectUpdate;

    return effect;
}

    

