#include "EnemyBullet.h"

#include "ObjectCommon.h"

void EnemyBulletUpdate(ObjectStruct* object);
void EnemyBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject);

extern s16 backgroundScrollSpeed;

ObjectStruct* EnemyBulletCreate(s16 x, s16 y, s16 speedx, s16 speedy, u16 damage, const GGAnimation* animation, s16 rectWidth, s16 rectHeight)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY_PROJECTILE,
                                                EnemyBulletUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                EnemyBulletCollide,
                                                NULL);

    if (!object)
    {
        return NULL;
    }

    object->speedx                  = speedx;
    object->speedy                  = speedy;

    s16 halfWidth = (rectWidth >> 1);
    s16 halfHeight = (rectHeight >> 1);

    SET_RECT_PIXEL(-halfWidth, halfWidth, -halfHeight, halfHeight);

    object->x                       = x - halfWidth;
    object->y                       = y - halfHeight;
    object->damage                  = damage;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, animation->vdpInfo->startIndex);


    SET_ANIMATION_POINTER(animation);

    return object;
}

void EnemyBulletUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("EnemyBullet Update");
#endif

    object->x += object->speedx;
    object->y += object->speedy;

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight << 1) < 0 ||
        object->screenx > 320 ||
        object->screeny + V2P(object->rectBottom << 1) < 0 ||
        object->screeny > 224)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void EnemyBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    object->healthPoints = 0;
    EffectCreate(&Riccochet, object->x, object->y - P2V(8), 0);
    ObjectManager_DestroyObject(object);
}

