#include "FlyingImp.h"

#include "ObjectCommon.h"

void FlyingImpSetup(ObjectStruct* object);
void FlyingImpUpdate(ObjectStruct* object);
void FlyingImpDraw(ObjectStruct* object);
void FlyingImpCollide(ObjectStruct* object, ObjectStruct* collidedObject);

//#define DEBUG_OBJECTS

#define FLYINGIMP_SPEED -4

u16 sinewaveTable[64] = 
{
    50,55,60,65,69,74,78,82,85,89,92,94,96,98,99,100,100,100,99,98,96,94,92,89,85,82,78,74,69,65,60,55,50,45,40,35,31,26,22,18,15,11,8,6,4,2,1,0,0,0,1,2,4,6,8,11,15,18,22,26,31,35,40,45,

};

ObjectStruct* FlyingImpCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                FlyingImpUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                FlyingImpCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("FlyingImp could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = FLYINGIMP_SPEED;
    object->speedy                  = 0;
    object->damage                  = 1;
    object->healthPoints            = 2;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, FlyingImp.vdpInfo->startIndex);
    object->lifetime = 0;
    object->extra1 = y;

    SET_RECT_PIXEL(4, 20, 2, 19);
    SET_ANIMATION(FlyingImp);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("FlyingImpCreate");
#endif

    return object;
}

void FlyingImpUpdate(ObjectStruct* object)
{
    u16 sinewaveTableValue = sinewaveTable[object->lifetime & 63];

    object->x += object->speedx;
    object->y = object->extra1 + sinewaveTableValue;

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    if (sinewaveTableValue < 50)
    {
        object->currentAnimationFrame = *(object->animation->frames);
    }
    else
    {
        object->currentAnimationFrame = *(object->animation->frames + 1);
    }
}

void FlyingImpDrop(ObjectStruct* object)
{
    object->speedy++;
    object->y += object->speedy >> 1;
    object->x += object->speedx;

    UPDATE_SCREEN_POSITION;

    if (!(object->lifetime & 7))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &SmallExplosion);
    }

    if (object->screeny > 224 ||
        object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
    }
    else if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        ObjectManager_DestroyObject(object);
    }
}

void FlyingImpCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->healthPoints == 0)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        StarPickupCreate(object->x - P2V(4), object->y - P2V(4));
        ObjectManager_DestroyObject(object);
    }
}

