#include "HomeBoy.h"

#include "ObjectCommon.h"

void HomeBoySetup(ObjectStruct* object);
void HomeBoyUpdate(ObjectStruct* object);
void HomeBoyDraw(ObjectStruct* object);
void HomeBoyCollide(ObjectStruct* object, ObjectStruct* collidedObject);

//#define DEBUG_OBJECTS

ObjectStruct* HomeBoyCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                HomeBoyUpdate, 
                                                HomeBoyDraw,
                                                HomeBoyCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("HomeBoy could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = 0;
    object->speedy                  = 0;
    object->damage                  = 1;
    object->healthPoints            = 150;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, HomeBoy.vdpInfo->startIndex);
    object->lifetime = 0;

    SET_RECT_PIXEL(0, 0, 0, 0);
    SET_ANIMATION(HomeBoy);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("HomeBoyCreate");
#endif

    return object;
}

void HomeBoyUpdate(ObjectStruct* object)
{
    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    if (object->lifetime > 420)
    {
        object->lifetime = 0;
    }
    else if (object->lifetime == 400)
    {
        EnemyBulletCreate(object->x + P2V(16), object->y + P2V(32), -5, 0, 1, &EnemyBullet, 8, 10);
        EnemyBulletCreate(object->x + P2V(16), object->y + P2V(32), -4, -2, 1, &EnemyBullet, 8, 10);
        EnemyBulletCreate(object->x + P2V(16), object->y + P2V(32), -4, 2, 1, &EnemyBullet, 8, 10);
        EnemyBulletCreate(object->x + P2V(16), object->y + P2V(32), -3, -3, 1, &EnemyBullet, 8, 10);
        EnemyBulletCreate(object->x + P2V(16), object->y + P2V(32), -3, 3, 1, &EnemyBullet, 8, 10);
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void HomeBoyDestroy(ObjectStruct* object)
{
    UPDATE_SCREEN_POSITION;

    switch (object->lifetime)
    {
    case 10:
        ExplosionCreate(object->x, object->y, &LargeExplosion);
        break;
    case 20:
        ExplosionCreate(object->x + P2V(16), object->y + P2V(16), &LargeExplosion);
        break;
    case 30:
        ExplosionCreate(object->x + P2V(16), object->y, &LargeExplosion);
        break;
    case 40:
        ExplosionCreate(object->x, object->y + P2V(16), &LargeExplosion);
        break;
    }

    if (object->lifetime > 40)
    {
        StarPickupCreate(object->x + P2V(16), object->y + P2V(16));
        StarPickupCreate(object->x, object->y + P2V(16));
        StarPickupCreate(object->x + P2V(16), object->y);
        StarPickupCreate(object->x, object->y);

        ObjectManager_DestroyObject(object);
    }
}

void HomeBoyCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->healthPoints == 0)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);

        object->tileAttribute = TILE_ATTR_FULL(PAL0, 1, 0, 0, HomeBoy.vdpInfo->startIndex);
        object->updateFunction = HomeBoyDestroy;
        object->lifetime = 0;
        object->rectLeft = 0;
        object->rectRight = 0;
        object->rectTop = 0;
        object->rectBottom = 0;
    }
}

void HomeBoyDraw(ObjectStruct* object)
{
    if (object->lifetime > 400)
    {
        GGAnimation_DrawNoFlip(object->screenx, object->screeny, &HomeBoyMouth, 0, TILE_ATTR(PAL0, 0, 0, 0));
    }

    GGAnimation_DrawObjectNoFlip(object);
}

