#include "IceBlock.h"

#include "ObjectCommon.h"

void IceBlockUpdate(ObjectStruct* object);
void IceBlockCollide(ObjectStruct* object, ObjectStruct* collidedObject);
void IceBlockGrabbed(ObjectStruct* object, ObjectStruct* grabber);


extern s16 backgroundScrollSpeed;

ObjectStruct* IceBlockCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                IceBlockUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                IceBlockCollide,
                                                IceBlockGrabbed);

    if (!object)
    {
        return NULL;
    }

    object->speedx                  = 0;
    object->speedy                  = 0;

    SET_RECT_PIXEL(0, IceBlock.width - 1, 0, IceBlock.height - 1);

    object->x                       = x - (object->rectRight >> 1);
    object->y                       = y - (object->rectBottom >> 1);
    object->damage                  = 0;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, IceBlock.vdpInfo->startIndex);


    SET_ANIMATION(IceBlock);

    return object;
}

void IceBlockUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("IceBlock Update");
#endif

    object->x += object->speedx;
    object->y += object->speedy;

    UPDATE_SCREEN_POSITION;

    if (object->screenx < 320 &&
        object->screeny + V2P(object->rectBottom) < 224 &&
        object->screeny > 0)
    {
        if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
        {
            EffectCreate(&Riccochet, object->x, object->y - P2V(8), 0);
            ObjectManager_DestroyObject(object);
        }
    }
    else
    {
        ObjectManager_DestroyObject(object);
    }
}

void IceBlockCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    /*
    object->healthPoints = 0;
    EffectCreate(&Riccochet, object->x, object->y - P2V(8), 0);
    ObjectManager_DestroyObject(object);
    */
}

void IceBlockGrabbed(ObjectStruct* object, ObjectStruct* grabber)
{
    ThrowObjectCreate(object->animation, object->x, object->y, object->rectLeft, object->rectRight, object->rectTop, object->rectBottom);
    ObjectManager_DestroyObject(object);
}
