#include "MiniHomeBoy.h"

#include "ObjectCommon.h"

void MiniHomeBoySetup(ObjectStruct* object);
void MiniHomeBoyUpdate(ObjectStruct* object);
void MiniHomeBoyCollide(ObjectStruct* object, ObjectStruct* collidedObject);

//#define DEBUG_OBJECTS

ObjectStruct* MiniHomeBoyCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                MiniHomeBoyUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                MiniHomeBoyCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("MiniHomeBoy could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = 0;
    object->speedy                  = 0;
    object->damage                  = 1;
    object->healthPoints            = 3;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, MiniHomeBoy.vdpInfo->startIndex);
    object->lifetime = 0;

    SET_RECT_PIXEL(2, 21, 12, 31);
    SET_ANIMATION(MiniHomeBoy);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("MiniHomeBoyCreate");
#endif

    return object;
}


u16 GetAngle(ObjectStruct* object, s16* x, s16* y)
{
    s16 anglex;
    s16 angley;

    s16 tile_distance_x = VIRTUAL_TO_TILE(object->x - playerObject->x);
    s16 tile_distance_y = VIRTUAL_TO_TILE(object->y - playerObject->y);

    u16 x_is_negative = (tile_distance_x < 0);
    u16 y_is_negative = (tile_distance_y < 0);

    tile_distance_x = tile_distance_x < 0 ? -tile_distance_x : tile_distance_x;
    tile_distance_y = tile_distance_y < 0 ? -tile_distance_y : tile_distance_y;

    if (tile_distance_x > ANGLE_TABLE_WIDTH - 1 || tile_distance_y > ANGLE_TABLE_HEIGHT - 1)
    {
        return 0;
    }

    u16 angleValue = GET_ANGLE_VALUE(tile_distance_x, tile_distance_y);

    anglex = angleValue >> 8;
    angley = angleValue & 255;

    if (!x_is_negative)
    {
        anglex = -anglex;
    }

    if (!y_is_negative)
    {
        angley = -angley;
    }

    *x = anglex;
    *y = angley;

    return 1;
}


void MiniHomeBoyUpdate(ObjectStruct* object)
{
    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    if (object->lifetime > 140)
    {

        s16 x = 0;
        s16 y = 0;
        if (GetAngle(object, &x, &y))
        {
            EnemyBulletCreate(object->x + P2V(0), object->y + P2V(16), x, y, 1, &EnemyBullet, 8, 10);
            object->lifetime = 0;
        }
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void MiniHomeBoyCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->healthPoints == 0)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x, object->y, &LargeExplosion);
        StarPickupCreate(object->x, object->y);
        ObjectManager_DestroyObject(object);
    }
}

