#include "StarPickup.h"

#include "ObjectCommon.h"
#include "..\WeaponManager.h"

void StarPickupUpdate(ObjectStruct* object);
void StarPickupDraw(ObjectStruct* object);
void StarPickupCollide(ObjectStruct* object, ObjectStruct* collidedObject);

extern s16 backgroundScrollSpeed;

ObjectStruct* StarPickupCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PICKUP,
                                                StarPickupUpdate, 
                                                StarPickupDraw,
                                                StarPickupCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("StarPickup could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL1, 1, 0, 0, StarPickup.vdpInfo->startIndex);
    object->extra1 = y;
    object->lifetime = 0;

    SET_ANIMATION(StarPickup);
    SET_RECT_PIXEL(0, 24, 0, 24);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("StarPickupCreate");
#endif

    return object;
}

extern u16 sinewaveTable[64];

void StarPickupUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("StarPickup Update");
#endif

    object->y = object->extra1 + sinewaveTable[object->lifetime & 63];

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0 ||
        object->lifetime == 640)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void StarPickupDraw(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("StarPickupDraw");
#endif

    if (object->lifetime < 480)
    {
        GGAnimation_DrawObjectNoFlip(object);
    }
    else
    {
        if (object->lifetime & 3)
        {
            GGAnimation_DrawObjectNoFlip(object);
        }
    }
}

void StarPickupCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (collidedObject == playerObject)
    {
        WeaponManager_AddStars(5);
        ObjectManager_DestroyObject(object);
    }
}

