#include "Swooper.h"

#include "ObjectCommon.h"

void SwooperUpdate(ObjectStruct* object);
void SwooperDraw(ObjectStruct* object);
void SwooperCollide(ObjectStruct* object, ObjectStruct* collidedObject);

//#define DEBUG_OBJECTS

#define DAMAGED_TIMER_COUNT 8

ObjectStruct* SwooperCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                SwooperUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                SwooperCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("Swooper could not be created!");
#endif
        return NULL;
    }

    object->x                       = x - P2V(SCREEN_WIDTH + 32);
    object->y                       = y;
    object->speedx                  = 15;
    object->speedy                  = 1;
    object->damage                  = 1;
    object->healthPoints            = 2;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, Swooper.vdpInfo->startIndex);
    object->lifetime = 0;

    SET_RECT_PIXEL(0, 0, 0, 0);
    SET_ANIMATION(Swooper);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("SwooperCreate");
#endif

    return object;
}

void SwooperUpdate(ObjectStruct* object)
{
    object->x += object->speedx;
    object->y += object->speedy;

    UPDATE_SCREEN_POSITION;

    if (object->lifetime < 95)
    {
        object->currentAnimationFrame = *(object->animation->frames);
    }
    else if (object->lifetime < 105)
    {
        object->currentAnimationFrame = *(object->animation->frames + 1);
        object->speedx = 0;
    }
    else if (object->lifetime == 105)
    {
        SET_RECT_PIXEL(2, 29, 4, 20);
    }
    else
    {
        object->currentAnimationFrame = *(object->animation->frames + 2);
        object->speedx = -15;

        if (object->screenx + V2P(object->rectRight)< 0)
        {
            ObjectManager_DestroyObject(object);
            return;
        }
    }
}

void SwooperDrop(ObjectStruct* object)
{
    object->speedy++;
    object->y += object->speedy >> 1;
    object->x += object->speedx;

    UPDATE_SCREEN_POSITION;

    if (!(object->lifetime & 7))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &SmallExplosion);
    }

    if (object->screeny > 224 ||
        object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
    }
    else if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        ObjectManager_DestroyObject(object);
    }
}

void SwooperCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (object->healthPoints == 0)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    if (object->healthPoints <= 0)
    {
        ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
        StarPickupCreate(object->x - P2V(4), object->y - P2V(4));
        ObjectManager_DestroyObject(object);
    }
}

