#include "ThrowObject.h"

#include "ObjectCommon.h"

void ThrowObjectUpdate(ObjectStruct* object);
void ThrowObjectUpdateThrown(ObjectStruct* object);
void ThrowObjectPull(ObjectStruct* object);
void ThrowObjectCollide(ObjectStruct* object, ObjectStruct* collidedObject);

ObjectStruct* ThrowObjectCreate(const GGAnimation* animation, s16 x, s16 y, s16 left, s16 right, s16 top, s16 bottom)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLAYER_PROJECTILE,
                                                      ThrowObjectPull, 
                                                      GGAnimation_DrawObjectNoFlip,
                                                      ThrowObjectCollide,
                                                      NULL);

    Print("throw object create");

    if (!object)
    {
        return NULL;
    }

    SET_RECT_VIRTUAL(left, right, top, bottom);

    object->x                       = 0;
    object->y                       = 0;
    object->damage                  = THROW_OBJECT_DAMAGE;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, StarProjectile.vdpInfo->startIndex);

    SET_ANIMATION(StarProjectile);

    grabbedObject = object;

    grabbedObject->extra1 = (u32)GrabbedEffectCreate(&GrabbedEffect, 0, 0, 0);

    return object;
}

void ThrowObjectPull(ObjectStruct* object)
{
    UPDATE_ANIMATION_FRAME_NOLOOP;
    UPDATE_SCREEN_POSITION;
}

void ThrowObjectUpdate(ObjectStruct* object)
{
    object->x = playerObject->x + P2V(32);
    object->y = playerObject->y  - P2V(4);//- ((object->rectBottom - object->rectTop) >> 1);

    UPDATE_ANIMATION_FRAME_NOLOOP;
    UPDATE_SCREEN_POSITION;
}

void ThrowObjectUpdateThrown(ObjectStruct* object)
{
    object->x += object->speedx;
    object->y += object->speedy;

    UPDATE_SCREEN_POSITION;

    if (object->screenx < 320 &&
        object->screeny < 224 &&
        object->screeny + V2P(object->rectBottom) > 0)
    {
        if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
        {
            ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
            ObjectManager_DestroyObject(object);
        }
    }
    else
    {
        ObjectManager_DestroyObject(object);
    }
}

void ThrowObjectCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    /*
    ExplosionCreate(object->x - P2V(4), object->y - P2V(4), &LargeExplosion);
    ObjectManager_DestroyObject(object);
    */
}



void ThrowGrabbedObject()
{
    grabbedObject->updateFunction = ThrowObjectUpdateThrown;

    // forward -> forward
    // backwards -> forward
    // backwards and up -> up-forward
    // forwards and up -> up-forward
    // backwards and down -> down-forward
    // forwards and down -> down-forward
    // up  (x == 0) -> up
    // down (x == 0) -> down 

    if (playerObject->speedy > 0) // down
    {
        if (playerObject->speedx > 0) // down right diagonal 
        {
            grabbedObject->speedx = 11;
            grabbedObject->speedy = 11;
        }
        else // left. straight down.
        {
            grabbedObject->speedx = 0;
            grabbedObject->speedy = 16;
        }
    }
    else if (playerObject->speedy < 0) // up
    {
        if (playerObject->speedx > 0) // up right diagonal 
        {
            grabbedObject->speedx = 11;
            grabbedObject->speedy = -11;
        }
        else // left. straight up.
        {
            grabbedObject->speedx = 0;
            grabbedObject->speedy = -16;
        }
    }
    else // only left and right. shoot forwards.
    {
        grabbedObject->speedx = 16;
        grabbedObject->speedy = 0;
    }

    ObjectManager_DestroyObject((ObjectStruct*)grabbedObject->extra1);
    grabbedObject->extra1 = NULL;

    grabbedObject = NULL;
}
