#include "WandPickup.h"

#include "ObjectCommon.h"
#include "..\WeaponManager.h"

void WandPickupUpdate(ObjectStruct* object);
void WandPickupDraw(ObjectStruct* object);
void WandPickupCollide(ObjectStruct* object, ObjectStruct* collidedObject);

extern s16 backgroundScrollSpeed;

ObjectStruct* WandPickupCreate(s16 x, s16 y)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PICKUP,
                                                WandPickupUpdate, 
                                                WandPickupDraw,
                                                WandPickupCollide,
                                                NULL);

    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("WandPickup could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, WandPickup.vdpInfo->startIndex);
    object->extra1 = 0;

    SET_ANIMATION(WandPickup);
    SET_RECT_PIXEL(0, 24, 0, 24);

#ifdef DEBUG_OBJECTS
    KDebug_Alert("WandPickupCreate");
#endif

    return object;
}

void WandPickupUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("WandPickup Update");
#endif

    //object->x += object->speedx;

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;
}

void WandPickupDraw(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("WandPickupDraw");
#endif

    GGAnimation_DrawObjectNoFlip(object);

    GGAnimation_DrawNoFlip(object->screenx + 8, object->screeny, &WandPickup, (object->extra1 >> 2), TILE_ATTR(PAL0, 1, 0, 0));
}

void WandPickupCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (collidedObject == playerObject)
    {
        WeaponManager_UpgradeWeapon((object->extra1 >> 2));
        ObjectManager_DestroyObject(object);
    }

    if (collidedObject->objectType == TYPE_PLAYER_PROJECTILE)
    {
        s16 newPosition = object->x + 30;

        if (!IsTileSolid_Virtual(newPosition + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
        {
            object->x = newPosition;
        }

        object->extra1++;
        if ((object->extra1 >> 2) > 1/*NUM_WEAPONS - 1*/) // four shots to change weapon
        {
            object->extra1 = 0;
        }

        //object->currentAnimationFrame = *(object->animation->frames + (object->extra1 >> 2));
    }

}

