#include "WandPickupContainer.h"

#include "ObjectCommon.h"
#include "..\WeaponManager.h"

void WandPickupContainerUpdate(ObjectStruct* object);
void WandPickupContainerCollide(ObjectStruct* object, ObjectStruct* collidedObject);

extern s16 backgroundScrollSpeed;

#define DAMAGED_TIMER_COUNT 8

ObjectStruct* WandPickupContainerCreate(s16 x, s16 y, u32* properties, u16 numProperties)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PICKUP,
                                                WandPickupContainerUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                WandPickupContainerCollide,
                                                NULL);


    if (!object)
    {
#ifdef DEBUG_OBJECTS
        KDebug_Alert("WandPickupContainer could not be created!");
#endif
        return NULL;
    }

    object->x                       = x;
    object->y                       = y;
    object->speedx                  = 0;
    object->speedy                  = 0;
    object->healthPoints            = 4;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, WandPickupContainer.vdpInfo->startIndex);

    SET_RECT_PIXEL(16, 36, 1, 23);
    SET_ANIMATION(WandPickupContainer);


    object->lifetime = 0;
    if (numProperties > 0)
    {
        object->extra1 = (u32)((Path*)properties[0])->pathSegments;
    }
    else
    {
        object->extra1 = 0;
    }

    object->extra2 = 0;


#ifdef DEBUG_OBJECTS
    KDebug_Alert("WandPickupContainerCreate");
#endif

    return object;
}

void WandPickupContainerUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("WandPickupContainer Update");
#endif

    if (object->extra1 != 0)
    {
        PathSegment* pathSegment = (PathSegment*)object->extra1;

        object->x += pathSegment->speedx;
        object->y += pathSegment->speedy;

        if (!object->extra2)
        {
            if (pathSegment->time > 0)
            {
                if (object->lifetime > pathSegment->time)
                {
                    object->extra1 = (u32)(pathSegment += 1);

                    if (pathSegment->time == 0)
                    {
                        object->extra1 = 0;
                    }
                }
            }
        }
        else
        {
            object->extra2--;

            if (!object->extra2)
            {
                object->lifetime = 0;
            }
        }
    }
    else
    {
        //object->x += object->speedx;
    }

    UPDATE_SCREEN_POSITION;

    if (object->screenx + V2P(object->rectRight)< 0)
    {
        ObjectManager_DestroyObject(object);
        return;
    }

    UPDATE_ANIMATION_FRAME_LOOP;

    if (object->damagedTimer)
    {
        object->damagedTimer--;

        if (!object->damagedTimer)
        {
            object->tileAttribute = TILE_ATTR_FULL(PAL0, 1, 0, 0, WandPickupContainer.vdpInfo->startIndex);
        }
    }

}

void WandPickupContainerCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    if (collidedObject->objectType == TYPE_PLAYER_PROJECTILE)
    {
        object->healthPoints--;

        if (object->healthPoints <= 0)
        {
            ExplosionCreate(object->x + P2V(11), object->y - P2V(4), &LargeExplosion);
            WandPickupCreate(object->x + P2V(15), object->y);
            ObjectManager_DestroyObject(object);
        }
        else
        {
            object->damagedTimer = DAMAGED_TIMER_COUNT;
            object->tileAttribute = TILE_ATTR_FULL(PAL1, 1, 0, 0, WandPickupContainer.vdpInfo->startIndex);
        }
    }
}

