#include "YellowWandBullet.h"

#include "ObjectCommon.h"

void YellowWandBulletUpdate(ObjectStruct* object);
void YellowWandBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject);

extern s16 backgroundScrollSpeed;

ObjectStruct* YellowWandBulletCreate(s16 x, s16 y, s16 speedx, s16 speedy, u16 damage, const GGAnimation* animation, s16 rectWidth, s16 rectHeight)
{
    ObjectStruct* object = ObjectManager_CreateObject(TYPE_PLAYER_BULLET,
                                                YellowWandBulletUpdate, 
                                                GGAnimation_DrawObjectNoFlip,
                                                YellowWandBulletCollide,
                                                NULL);

    if (!object)
    {
        return NULL;
    }

    object->speedx                  = speedx;
    object->speedy                  = speedy;

    SET_RECT_PIXEL(0, rectWidth, 0, rectHeight);

    object->x                       = x - (object->rectRight >> 1);
    object->y                       = y - (object->rectBottom >> 1);
    object->damage                  = damage;
    object->healthPoints            = 1;
    object->tileAttribute           = TILE_ATTR_FULL(PAL0, 1, 0, 0, animation->vdpInfo->startIndex);


    SET_ANIMATION_POINTER(animation);

    return object;
}

void YellowWandBulletUpdate(ObjectStruct* object)
{
#ifdef DEBUG_OBJECTS
    KDebug_Alert("YellowWandBullet Update");
#endif

    object->x += object->speedx;
    object->y += object->speedy;

    UPDATE_SCREEN_POSITION;

    if (object->screenx < 320 &&
        object->screeny < 224 &&
        object->screeny + V2P(object->rectBottom) > 0)
    {
        if (IsTileSolid_Virtual(object->x + (object->rectRight >> 1), object->y + (object->rectBottom >> 1)))
        {
            EffectCreate(&Riccochet, object->x, object->y - P2V(8), 0);
            ObjectManager_DestroyObject(object);
        }
    }
    else
    {
        ObjectManager_DestroyObject(object);
    }
}

void YellowWandBulletCollide(ObjectStruct* object, ObjectStruct* collidedObject)
{
    object->healthPoints = 0;
    ObjectManager_DestroyObject(object);
}

