// gale2c.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "..\galefile091207\galefile.h"
#include <string>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <sstream>
#include <math.h>

std::string galeFilename;
std::string outputName;
std::string cFilename;
std::string headerFilename;
std::string frameArrayName;
std::string outputSpriteDataName;
std::string outputFolder;

DWORD numberOfFrames = 0;

DWORD tilesPerFrame = 0;
DWORD totalTiles = 0;
DWORD maxTilesPerFrame = 0;

BITMAP generalBitmapInfo;

LPVOID galefile = NULL;
std::ofstream cfile;
std::ofstream headerfile;
std::stringstream cfilecontent;
std::stringstream spritedata;

void EnsureDestinationDirectoryExists()
{
    if (outputFolder != "")
    {
        std::string systemCall;
        systemCall = "mkdir " + outputFolder;
        system(systemCall.c_str());
    }
}

void OutputHeader()
{
    //EnsureDestinationDirectoryExists();

    headerfile.open(outputFolder + headerFilename);

    std::string headerGuard = outputName + "_ANIMATION_INCLUDE_H";
    std::transform(headerGuard.begin(), headerGuard.end(), headerGuard.begin(), ::toupper);

    // header guard
    headerfile << "// This file is automatically generated by \n";
    headerfile << "// export_graphicsgale_animations.py\n";
    headerfile << "#ifndef " << headerGuard << "\n";
    headerfile << "#define " << headerGuard << "\n";
    headerfile << "\n";
    headerfile << "#include <genesis.h>\n";
    headerfile << "#include \"..\\..\\GGAnimationDraw.h\"\n";
    headerfile << "#include \"..\\..\\VDPInfo.h\"\n";
    headerfile << "\n";
    headerfile << "extern const GGAnimation " << outputName << ";\n"; 
    headerfile << "extern VDPInfo " << outputName << "VDPInfo;\n";  
    headerfile << "\n";
    // end header guard
    headerfile << "#endif\n\n";
    headerfile.close();
}

std::string BuildFrameName(int frameNumber)
{
    std::stringstream stringStream;
    stringStream << outputName << "Frame" << frameNumber;

    return stringStream.str();
}

std::string BuildSpriteArrayName(int frameNumber)
{
    std::stringstream stringStream;
    stringStream << outputName << "Frame" << frameNumber << "Sprites";

    return stringStream.str();
}

std::string BuildFrameSpriteName(int frameNumber, int spritenumber)
{
    std::stringstream stringStream;
    stringStream << outputName << "Frame" << frameNumber << "Sprite" << spritenumber;

    return stringStream.str();
}

DWORD SwitchEndian(DWORD value)
{
    unsigned char tempValue[4];
    tempValue[0] = (value >> 24) & 0xff;
    tempValue[1] = (value >> 16) & 0xff;
    tempValue[2] = (value >> 8) & 0xff;
    tempValue[3] = value & 0xff;

    DWORD tempDWORD = *(DWORD*)&tempValue;

    return (DWORD)tempDWORD;
}

std::string WriteDWORD(DWORD value)
{
    value = SwitchEndian(value);

    std::stringstream tempStringStream;

    tempStringStream << "0x";

    tempStringStream.width(8);
    tempStringStream.fill('0');
    tempStringStream << std::hex << value;

    return tempStringStream.str();
}

int tileIndex = 0;
int spriteDataWidth = 1;
int spriteDataHeight = 8;

void OutputTile(int startx, int starty, const DWORD* imagedata, int dataWidth)
{
    spritedata << "// tile: " << tileIndex << "\n";
    tileIndex++;
    for (int loop2 = spriteDataHeight - 1; loop2 >= 0; loop2--)
    {
        for (int loop = 0; loop < spriteDataWidth; loop++)
        {
            spritedata << "    " << WriteDWORD(imagedata[(loop + startx) + ((loop2 + starty) * dataWidth)]) << ",\n";
        }
    }
}


int OutputSprites(int frameNumber)
{
    tilesPerFrame = 0;

    HBITMAP bitmap = ggGetBitmap(galefile, frameNumber, 0);
    BITMAP bitmapInfo;
    GetObject(bitmap, sizeof(BITMAP), &bitmapInfo);
    const DWORD* imagedata = (const DWORD*)bitmapInfo.bmBits;
    // Graphics Gale exports the bitmap upside down. Fugh.


    int numSpritesOnX = bitmapInfo.bmWidth / 32 + ((bitmapInfo.bmWidth % 32) != 0);
    int numSpritesOnY = bitmapInfo.bmHeight / 32 + ((bitmapInfo.bmHeight % 32) != 0);
    int numberOfSprites = numSpritesOnX * numSpritesOnY;


    int dataWidth = (bitmapInfo.bmWidth / 8);
    int spriteDataWidth = 1;
    int spriteDataHeight = 8;

    int tileWidth = bitmapInfo.bmWidth / 8;
    int tileHeight = bitmapInfo.bmHeight / 8;
    int spriteIndex = 0;

    for (int spriteloopx = 0; spriteloopx < numSpritesOnX; spriteloopx++)
    {
        for (int spriteloopy = numSpritesOnY - 1; spriteloopy >= 0; spriteloopy--)
        {
            int tilestartx = spriteloopx * 4;
            int tilestarty = spriteloopy * 4;

            int tileendx = tilestartx + 4;
            int tileendy = tilestarty + 4;

            int tempWidth = 4;
            int tempHeight = 4;

            if (tileendx > tileWidth)
            {
                tempWidth = tempWidth - (tileendx - tileWidth);
                tileendx = tileWidth;
            }

            if (tileendy > tileHeight)
            {
                tempHeight = tempHeight - (tileendy - tileHeight);
                tileendy = tileHeight;
            }

            tilesPerFrame += tempWidth * tempHeight;

            int spriteDataSize = 4 *         // width of one line
                                 tempWidth * // number of lines on x
                                 8 *         // lines * tiles on y.
                                 tempHeight; // number of lines on y

            std::string spriteName = BuildFrameSpriteName(frameNumber, spriteIndex);
            std::string spriteDataName = spriteName + "Data";
            spriteIndex++;

            for (int tileloopx = tilestartx; tileloopx < tileendx; tileloopx++)
            {
                for (int tileloopy = (tileendy - 1); tileloopy >= tilestarty; tileloopy--)
                {
                    int startx = tileloopx * spriteDataWidth;
                    int starty = tileloopy * spriteDataHeight;
                    OutputTile(startx, starty, imagedata, dataWidth);
                }
            }

            cfilecontent << "const GGSprite " << spriteName << " = \n";
            cfilecontent << "{\n";
            cfilecontent << "    " << outputSpriteDataName << " + " << (8 * totalTiles) << ",\n";
            cfilecontent << "    " << tempWidth << ", // tile width\n";
            cfilecontent << "    " << tempHeight << ", // tile height\n";
            cfilecontent << "    SPRITE_SIZE(" << tempWidth << ", " << tempHeight << "), // sgdk sprite size\n";
            cfilecontent << "    " << spriteloopx * 32 << ", // x position offset\n";
            cfilecontent << "    " << bitmapInfo.bmHeight - ((spriteloopy * 32) + (tempHeight * 8)) << ", // y position offset\n";
            cfilecontent << "    " << totalTiles << ", // tile offset in sprite tile data\n";          
            cfilecontent << "};\n\n";

            totalTiles += tempWidth * tempHeight;
        }
    }

    return numberOfSprites;
}

void OutputSpriteArray(int frameNumber, int numberOfSprites)
{
    cfilecontent << "const GGSprite* const " << BuildSpriteArrayName(frameNumber) << "[" << numberOfSprites << "] = \n";
    cfilecontent << "{\n";

    for (int loop = 0; loop < numberOfSprites; loop++)
    {
        cfilecontent << "    &" << BuildFrameSpriteName(frameNumber, loop) << ",\n";
    }

    cfilecontent << "};\n\n";
}

void OutputFrame(int frameNumber)
{
    int numberOfSprites = OutputSprites(frameNumber);

    OutputSpriteArray(frameNumber, numberOfSprites);

    cfilecontent << "\n";
    cfilecontent << "const GGFrame " << BuildFrameName(frameNumber) << " = \n";
    cfilecontent << "{\n";
    cfilecontent << "    " << BuildSpriteArrayName(frameNumber) << ",\n";
    cfilecontent << "    " << numberOfSprites << ", // number of sprites\n";
    cfilecontent << "};\n";
    cfilecontent << "\n";
}

void OutputFrames()
{
    tileIndex = 0;
    spritedata << "u32 const " << outputSpriteDataName << "[] = \n";
    spritedata << "{\n";

    // individual frames
    for (DWORD loop = 0; loop < numberOfFrames; loop++)
    {
        OutputFrame(loop);

        if (tilesPerFrame > maxTilesPerFrame)
        {
            maxTilesPerFrame = tilesPerFrame;
        }
    }

    spritedata << "};\n\n";
}

void OutputFrameArray()
{
    cfilecontent << "const GGFrame* const " << frameArrayName << "[" << numberOfFrames << "] = \n";
    cfilecontent << "{\n";

    for (DWORD loop = 0; loop < numberOfFrames; loop++)
    {
        cfilecontent << "    &" << BuildFrameName(loop) << ",\n";
    }

    cfilecontent << "};\n\n";
}

static inline float round(float val)
{
    return floor(val + 0.5f);
}

void OutputAnimationStruct()
{
    // final struct
    cfilecontent << "\n";
    //const TileMap testmap_background = 

    LONG frameDelayTime = ggGetFrameInfo(galefile, 0, 2); // get only the frame delay time of the first frame
    frameDelayTime = (LONG)(round((float)frameDelayTime / 17.0f)); // 17 ms per frame

    cfilecontent << "\n";
    cfilecontent << "VDPInfo " << outputName << "VDPInfo;\n";   
    cfilecontent << "\n";

    cfilecontent << "const GGAnimation " << outputName << " = \n";
    cfilecontent << "{\n";
    cfilecontent << "    " << frameArrayName << ",\n";
    cfilecontent << "    " << numberOfFrames << ", // number of frames\n";
    cfilecontent << "    " << frameDelayTime << ", // frame delay time,\n";
    cfilecontent << "    " << generalBitmapInfo.bmWidth << ", // width in pixels\n";
    cfilecontent << "    " << generalBitmapInfo.bmHeight << ", // height in pixels\n";
    cfilecontent << "    " << maxTilesPerFrame << ", // max tiles per frame\n";
    cfilecontent << "    " << totalTiles << ", // the total number of tiles in the animation\n";
    cfilecontent << "    " << outputSpriteDataName << ", // start of the sprite data\n";
    cfilecontent << "    &" << outputName << "VDPInfo, // vdp info\n";

    cfilecontent << "};\n"; 
}

void OutputCFile()
{
    numberOfFrames = ggGetFrameCount(galefile);
    HBITMAP bitmap = ggGetBitmap(galefile, 0, 0);

    OutputFrames();
    OutputFrameArray();
    OutputAnimationStruct();

    cfile.open(outputFolder + cFilename);

    // include genesis.h
    cfile << "#include <genesis.h>\n";

    // include header
    cfile << "#include \"" << headerFilename << "\"\n";
    cfile << "\n";
    cfile << "\n";

    cfile << spritedata.str();
    cfile << cfilecontent.str();
    cfile.close();

}

void BuildNames()
{
    std::size_t index = galeFilename.rfind(".");
    outputName = galeFilename.substr(0, index);
    index = outputName.rfind("\\");
    if (index != std::string::npos)
        outputName = outputName.substr(index + 1);
    cFilename = outputName + ".c";
    headerFilename = outputName + ".h";
    frameArrayName = outputName + "Frames";
    outputSpriteDataName = outputName + "spriteData";
}

int main(int argc, char* argv[])
{
    try
    {
        if (argv[1] == NULL)
        {
            throw -6;
        }

        if (argv[2] != NULL)
        {
            outputFolder = argv[2];
            outputFolder += "\\";
        }

        galeFilename = argv[1];

        galefile = ggOpen(galeFilename.c_str());

        if (galefile == NULL)
        {
            throw -1;
        }

        DWORD numberOfFrames = ggGetFrameCount(galefile);

        if (numberOfFrames == 0)
        {
            throw -2;
        }

        HBITMAP bitmap = ggGetBitmap(galefile, 0, 0);

        if (bitmap == NULL)
        {
            throw -3;
        }

        if (GetObject(bitmap, sizeof(BITMAP), &generalBitmapInfo) == 0)
        {
            throw -4;
        }

        if (generalBitmapInfo.bmBitsPixel != 4)
        {
            throw -5;
        }

        BuildNames();
        OutputHeader();
        OutputCFile();
    }
    catch (int errorCode)
    {
        switch (errorCode)
        {
        case -1:
            printf("Error: File not found or error loading\n");
            break;
        case -2:
            printf("No frames found in file\n");
            break;
        case -3:
            printf("Error retrieving bitmap data");
            break;
        case -4:
            printf("BitmapInfo is NULL\n");
            break;
        case -5:
            printf("Bitmap data is not 4 bits per pixel\n");
            break;
        case -6:
            printf("No Graphics Gale file specified\n");
            printf("\ngale2c.exe [inputfile] [optional_destination_folder]\n");
            break;
        }
    }

    if (galefile != NULL)
    {
        ggClose(galefile);
    }

    return 0;
}

