# helper script that generates .h and .c files from tiled map files

SECTION_PIXEL_SIZE = 16
SECTION_TILE_SIZE = SECTION_PIXEL_SIZE / 8

import sys
import os
import time
import math
import xml.etree.ElementTree as ET
from collections import defaultdict

#changing the destination folder requires updating the SGDK makefile too
destination_folder = 'exported\\maps'
destination_path = os.path.realpath('.') + "\\" + destination_folder

tilesetbitmapfilenames = set()

playerStartX = 0
playerStartY = 0
mapwidth = 0
mapheight = 0
maptype = 'none'
vdpwidth = 0
vdpheight = 0
tilemapdata = 0
objectgroup = None
vpdassetlistdict = {}
assetrefs = {}
editorobjects = []
tilesetfilename = ""
numSpawns = 0
paths = {}
isforegroundmap = 0

def gettilesetfilename(tilesetnode):
    global tilesetfilename
    sourcefilename = tilesetnode.get('source')

    currentdirectory = os.getcwd()
    os.chdir('..\\gamedata\\tilesets')
    
    tree = ET.parse(sourcefilename)
    root = tree.getroot()
    imagenode = root.find('image')
    tilesetfilename = imagenode.get('source')
    tilesetfilename = tilesetfilename.rsplit('.', 1)[0]
    os.chdir( currentdirectory)
    

def importpaths():
    global objectgroup

    if objectgroup is None:
        return

    for object in objectgroup:
        objectid = object.get('objectid')
        if objectid is None:
            polyline = object.find('polyline')
            if polyline is not None:
                name = object.get('name')
                if name in paths:
                    print '#### ERROR: There are two or more Polylines with the same name' 
                    sys.exit()
                paths[name] = object
    

def importmap(filename):
    global mapwidth
    global mapheight
    global tilemapdata
    global objectgroup

    tree = ET.parse(filename)
    root = tree.getroot()
    mapwidth = int(root.get('width'))
    mapheight = int(root.get('height'))
    tilemapdata = root.find('layer').find('data').text
    tilemapdata = tilemapdata.split(',')
    gettilesetfilename(root.find('tileset'))
    objectgroup = root.find('objectgroup')

    importpaths()

def computevdpsettings():
    global maptype
    global vdpwidth
    global vdpheight
    global mapwidth
    global mapheight

    if (mapwidth * mapheight <= 64 * 64) and (mapwidth <= 128 and mapheight <= 128):
        maptype = 'MAPTYPE_VDP'

        if mapwidth <= 32:
            vdpwidth = 32

            if mapheight <= 32:
                vdpheight = 32
            elif mapheight <= 64:
                vdpheight = 64
            elif mapheight <= 128:
                vdpheight = 128

        elif mapwidth <= 64:
            vdpwidth = 64

            if mapheight <= 32:
                vdpheight = 32
            elif mapheight <= 64:
                vdpheight = 64

        elif mapwidth <= 128:
            vdpwidth = 128

            if mapheight <= 32:
                vdpheight = 32


    else:
        vdpwidth = 64
        vdpheight = 32

        if (mapwidth > 128 and mapheight > 64):
            maptype = 'MAPTYPE_8WAY'
        elif mapwidth > 128 and mapheight <= 64:
            maptype = 'MAPTYPE_HORIZONTAL'
            if mapheight > 32:
                vdpheight = 64;
        elif mapwidth <= 128:
            maptype = 'MAPTYPE_VERTICAL'
            if mapwidth > 64:
                vdpwidth = 128
            elif mapwidth <= 32:
                vdpwidth = 32

def loadAssetListsXml():
    global vpdassetlistdict
    global assetrefs

    tree = ET.parse('..\gamedata\GameObjectLibrary\GameObjectLibrary.xml')
    root = tree.getroot()

    objects = root.find('objects')
    if objects == None:
        return

    for object in objects :
        nodetype = object.tag
        objecttype = object.get('type')

        if nodetype == 'object':
            assets = object.get('assets')
            if assets is not None:
                vpdassetlistdict[objecttype] = assets.split()
        if nodetype == 'editorobject':
            editorobjects.append(objecttype)

        refs =object.get('assetrefs')
        if refs is not None:
            assetrefs[objecttype] = refs.split()

def exportc_paths(outputmapname, exported_c):

    for name, object in paths.iteritems():

        path = object.find('polyline')

        #get points
        pointsxml = path.get('points').split(' ')
        points = []

        for point in pointsxml:
            points.append(point.split(','))

        #get speed
        speed = 1
        for property in object.find('properties'):
            if property.get('name') == 'Speed':
                speed = int(property.get('value'))
                if speed is None:
                    speed = 1

        pathSegmentsName = outputmapname + '_' + name + '_PathSegments'

        numPathSegments = len(points) - 1

        exported_c.write('const PathSegment const ' + pathSegmentsName + '[' + str(numPathSegments + 1) + '] = \n')
        exported_c.write('{\n')

        totalsteps = 0

        xspeed = 0.0
        yspeed = 0.0

        for loop in xrange(0, numPathSegments):
            point1 = points[loop]
            point2 = points[loop + 1]

            x1 = float(point1[0])
            y1 = float(point1[1])
            x2 = float(point2[0])
            y2 = float(point2[1])

            dist = math.hypot(x2 - x1, y2 - y1)

            xspeed = (x2 - x1) / dist * float(speed)
            yspeed = (y2 - y1) / dist * float(speed)

            numsteps = dist / speed
            totalsteps = totalsteps + numsteps

            exported_c.write('    {\n')
            exported_c.write('        ' + str(int(xspeed)) + ',\n')
            exported_c.write('        ' + str(int(yspeed)) + ',\n')
            exported_c.write('        ' + str(int((totalsteps * 4))) + ',\n')
            exported_c.write('    },\n')

        #sentinel ending value
        exported_c.write('    {\n')
        exported_c.write('        ' + str(int(xspeed)) + ',\n')
        exported_c.write('        ' + str(int(yspeed)) + ',\n')
        exported_c.write('        ' + str(int(0)) + ',\n')
        exported_c.write('    },\n')

        exported_c.write('};\n\n')

        exported_c.write('const Path ' + outputmapname + '_' + name + ' = \n')
        exported_c.write('{\n')
        exported_c.write('    ' + pathSegmentsName + ',\n')
        exported_c.write('    ' + str(numPathSegments + 1) + '\n')
        exported_c.write('};\n\n')


def exportc_vdpassetfunction(outputmapname, exported_c):

    exported_c.write('void ' + outputmapname + '_LoadVDPAssets(const VPMap* thismap)\n{\n')

    if objectgroup is not None:
        global vpdassetlistdict

        exportedobjecttypes = set();
        for child in objectgroup:
            objectid = child.get('objectid')
            if objectid is not None:
                objectid = int(objectid)
                exportedobjecttypes.add(child.get('objecttype'))

        exportedassets = set()

        for objecttype in exportedobjecttypes:
            if objecttype in vpdassetlistdict:
                vdpassetlist = vpdassetlistdict[objecttype]
                if vdpassetlist[0] != 'None':
                    for vdpasset in vdpassetlist:
                        exportedassets.add(vdpasset)
            else:
                if (objecttype not in editorobjects):
                    exportedassets.add(objecttype)

        for objecttype in exportedobjecttypes:
            if objecttype in assetrefs:
                for vdpasset in assetrefs[objecttype]:
                    if vdpasset in vpdassetlistdict:
                        vdpassetlist = vpdassetlistdict[vdpasset]
                        if vdpassetlist[0] != 'None':
                            for vdpasset in vdpassetlist:
                                exportedassets.add(vdpasset)
                    else:
                        if (vdpasset not in editorobjects):
                            exportedassets.add(vdpasset)

        exportedassets = sorted(exportedassets)

        for exportedasset in exportedassets:
            exported_c.write('    LOAD_GGANIMATION_TO_VDP(' + exportedasset + ');\n')

    exported_c.write('''\n    VDPManager_LoadSpriteDataToVDP(thismap->tilesetSprite, 
                                   thismap->tilesetVDPInfo);\n\n''')

    exported_c.write('}\n\n')

def exportc_objectspawnorder(outputmapname, exported_c):
    global numSpawns

    if objectgroup is None:
        return

    spawnlist = defaultdict(set)

    uniqueNumber = 1

    for child in objectgroup:
        objectid = child.get('objectid')
        if objectid is not None:
            yposition = child.get('y')
            xposition = int(child.get('x'))
            objecttype = child.get('objecttype')
            parentName = child.get('parentName')
            if objecttype != 'Player':

                exportedProperties = list()
                name = child.get('name')
                exportedPropertiesName = 'NULL'

                if parentName != '':
                    exportedProperties.append('(u32)&' + outputmapname + '_' + parentName)

                properties = child.find('properties')
                if properties is not None:
                    for property in properties:
                        value = property.get('value')
                        if value is '':
                            value = 0
                        exportedProperties.append(value)

                if exportedProperties:
                    exportedPropertiesName = outputmapname + '_' + name + '_properties' + str(uniqueNumber)

                    uniqueNumber = uniqueNumber + 1

                    exported_c.write('u32 const ' + exportedPropertiesName + '[] =\n')
                    exported_c.write('{\n')
                    for property in exportedProperties:
                        exported_c.write('    ' + str(property) + ',\n')
                    exported_c.write('};\n\n');

                tuple = (xposition, yposition, objecttype, exportedPropertiesName, str(len(exportedProperties)))
                spawnlist[xposition].add(tuple)


    exported_c.write('SpawnStruct const ' + outputmapname + '_ObjectSpawnList[] =\n')
    exported_c.write('{\n')

    numSpawns = 0


                

    for key in sorted(spawnlist.iterkeys()):
        for spawn in spawnlist[key]:
            numSpawns = numSpawns + 1
            exported_c.write('    {' + spawn[2] + 'Create, PIXEL_TO_VIRTUAL(' + str(spawn[0]) + '), PIXEL_TO_VIRTUAL(' + spawn[1] + '), ')
            
            if spawn[3] == "NULL":
                exported_c.write(spawn[3] + ', ')
            else:
                exported_c.write('(u32*)' + spawn[3] + ', ')

            exported_c.write(spawn[4])

            exported_c.write('},\n')

    exported_c.write('    {0, NULL, 0, 0}\n')

    exported_c.write('};\n\n')

def exportcfile(outputname, destinationfilename):
    global mapwidth
    global mapheight
    global maptype
    global vdpwidth
    global vdpheight
    global tilemapdata
    global objectgroup
    global tilesetfilename
    global playerStartX
    global playerStartY
    global isforegroundmap

    exported_c = open(destinationfilename, "w")
    exported_c.write("// **************************************************** \n")
    exported_c.write("// This file is automatically generated by\n")
    exported_c.write("// export_tiled_maps.py\n")
    exported_c.write("// **************************************************** \n\n")    
    
    exported_c.write('#include "' + outputname + '.h"\n\n')
    exported_c.write('#include "..\..\MathUtils.h"\n')
    exported_c.write('#include "..\..\VDPManager.h"\n')
    exported_c.write('#include "..\..\Path.h"\n')
    exported_c.write('#include "..\..\PlayerManager.h"\n')
    exported_c.write('#include "..\..\MathUtils.h"\n')
    exported_c.write('#include "..\..\exported\ExportedGGAnimations.h"\n')
    exported_c.write('#include "..\..\exported\ExportedGenResSprites.h"\n')
    exported_c.write('#include "..\..\exported\ExportedTileAttributes.h"\n')
    exported_c.write('#include "..\..\generated\AllObjects.h"\n')
    exported_c.write('#include "..\..\VDPManager.h"\n')
    exported_c.write("\n")

    exported_c.write("unsigned short const " + outputname + "_tilemapdata[" + str(mapwidth * mapheight) + "] = // " + str(mapwidth) + " * " + str(mapheight) + "\n")
    exported_c.write("{\n")
    contents = ""
 
    for heightcounter in range(0, mapheight):
        for widthcounter in range(0, mapwidth):
            tile = int(tilemapdata[widthcounter + (heightcounter * mapwidth)])

            if tile > 0:
                tile = tile - 1

            if isforegroundmap == 1:
                tile = tile | (1 << 15)

            contents = contents + str(tile)
            contents = contents + ","
        contents = contents + "\n"
    exported_c.write(contents)
    exported_c.write("\n};\n\n")

    playerStartX = 0
    playerStartY = 0

    exportc_vdpassetfunction(outputname, exported_c)
    exportc_paths(outputname, exported_c)
    exportc_objectspawnorder(outputname, exported_c)

    exported_c.write('const VPMap ' + outputname + ' = \n')
    exported_c.write('{\n')
    exported_c.write('    ' + outputname + '_LoadVDPAssets,\n')

    if objectgroup is None:
        exported_c.write('    NULL, // object spawn list\n')
        exported_c.write('    0, // num spawns in spawn list\n')
    else:
        exported_c.write('    ' + outputname + '_ObjectSpawnList,    // object spawn list\n')
        exported_c.write('    ' + str(numSpawns) + ',    // num spawns in spawn list\n')

    exported_c.write('    ' + str(mapwidth) + ',    // map width\n')
    exported_c.write('    ' + str(mapheight) + ',    // map height\n')

    exported_c.write('    ' + str(maptype) + ',    // map type\n')
    exported_c.write('    ' + str(vdpwidth) + ',    // vdp plane width\n')
    exported_c.write('    ' + str(vdpheight) + ',    // vdp plane height\n')

    exported_c.write('    ' + outputname + '_tilemapdata,\n')
    exported_c.write('    &' + tilesetfilename + 'Sprite,\n')
    exported_c.write('    &' + tilesetfilename + 'VDPInfo,\n')


    if objectgroup is None:
        exported_c.write('    NULL,\n')
    else:
        exported_c.write('    ' + tilesetfilename + 'TileAttributes,\n')
    
    exported_c.write('    PIXEL_TO_VIRTUAL(' + str(playerStartX) + '), // player start x\n')
    exported_c.write('    PIXEL_TO_VIRTUAL(' + str(playerStartY) + '), // player start y\n')
    exported_c.write('}; \n\n')
    exported_c.close()

def exporthfile(outputname, destinationfilename):
    exported_h = open(destinationfilename, "w")

    exported_h.write("// **************************************************** \n")
    exported_h.write("// This file is automatically generated by\n")
    exported_h.write("// export_gentiled_maps.py\n")
    exported_h.write("// **************************************************** \n\n")
    exported_h.write('#ifndef ' + outputname.upper() + '_MAP_INCLUDE_H\n')
    exported_h.write('#define ' + outputname.upper() + '_MAP_INCLUDE_H\n\n')
    exported_h.write('#include "..\..\Map.h"\n\n')
    exported_h.write('extern const VPMap ' + outputname + ';\n')
    exported_h.write('\n#endif\n')
    exported_h.close()  



from glob import glob
print "************** Exporting Tiled Maps To Code **************"
filenames = glob('..\gamedata\Maps\*.gmx')
    
loadAssetListsXml()

includes=[]
    
for filename in filenames:
    sourcefilename = os.path.realpath('.') + "\\" + filename

    isforegroundmap = 1

    if 'background' in filename.lower():
        isforegroundmap = 0

    outputname = os.path.splitext(filename)[0]
    outputname = os.path.basename(outputname)   
    destinationcfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.c'
    destinationhfilename = os.path.realpath('.') + '\\' + destination_folder + '\\' + outputname + '.h'

    includes.append(outputname + ".h")
    
    docopy = False
    
    if (os.path.exists(destinationcfilename) == False):
        #print destinationcfilename + " is missing. Copying"
        docopy = True
    
    if docopy == False:
        sourcetime = os.path.getmtime(sourcefilename)
        destinationtime = os.path.getmtime(destinationcfilename)

        if sourcetime > destinationtime:
            #print "sourcefilename is newer than destination filename"
            docopy = True

    if docopy == True:
        print "Exporting " + filename + " to " + destination_folder

        if not os.path.exists(destination_path):
            os.makedirs(destination_path)
    
        importmap(filename)
        computevdpsettings()
        exportcfile(outputname, destinationcfilename)
        exporthfile(outputname, destinationhfilename)
    
    
exported_h = open("exported\\ExportedMaps.h", "w")
exported_h.write("// **************************************************** \n")
exported_h.write("// This file is automatically generated by\n")
exported_h.write("// export_gentiled_maps.py\n")
exported_h.write("// **************************************************** \n\n")
exported_h.write('#ifndef EXPORTED_GENTILED_MAPS_H\n')
exported_h.write('#define EXPORTED_GENTILED_MAPS_H\n\n')

for include in includes:
    exported_h.write('#include "maps\\' + include + '"\n')

exported_h.write('\n#endif\n')
exported_h.close()
    
