# helper script that generates .h and .c files for exported sprites in .rc files

import sys

print "************** Generating Debug Settings **************"

destination_folder = 'generated\\'

debugsettings = [
    #setting_name,          type,           min,    max,    default,    description
    ["collisions",          "bool",         0,      1,      1,          "Collisions",           "NULL"],
    ["debugmovement",       "bool",         0,      1,      1,          "Debug Movement",       "NULL"],
    ["debugmovementspeed",  "int",          1,      20,     1,          "Debug Movement Speed", "NULL"],
    ["waitforvsync",        "bool",         0,      1,      1,          "Wait for VSync",       "NULL"],
    ["showFPSCounter",      "bool",         0,      1,      0,          "Show FPS Counter",     "NULL"],
    ["showCollisionBox",    "bool",         0,      1,      0,          "Show Collision box",   "NULL"],
    ["invincible",          "bool",         0,      1,      0,          "Invincible",           "NULL"],
    ["drawSwordDamage",     "bool",         0,      1,      0,          "Show Sword Damage",    "NULL"],
    ["testvalue",           "int",          10,     20,     10,         "Test Value",           "NULL"],
    ["level",               "stringarray",  0,      0,      0,          "Level Select",         "LevelManager_SwitchToLevel",   "*currentValue",    "levelNames", "NUM_LEVELS"],
    ["restartlevel",        "function",     0,      0,      0,          "Restart Level",        "LevelManager_RestartLevel",   ""]
]

#************ generate DebugSettings.h ************

file = open(destination_folder + "DebugSettings.h", "w")

file.write("// ****************************************************************** \n")
file.write("// This file is automatically generated by generate_debug_settings.py\n")
file.write("// ****************************************************************** \n\n")
file.write("#ifndef DEBUG_SETTINGS_INCLUDE_H\n")
file.write("#define DEBUG_SETTINGS_INCLUDE_H\n\n")
file.write("#include <genesis.h>\n\n")
file.write("typedef struct \n{\n")

for row in debugsettings:
    file.write("    u16 " + row[0] + ";\n")

file.write("} DebugSettingsStruct;\n\n")
file.write("extern DebugSettingsStruct debugSettings;\n\n")
file.write("void DebugSettings_Init();\n\n")
file.write("#endif\n\n")

file.close()

#************ generate DebugSettings.c ************

file = open(destination_folder + "DebugSettings.c", "w")

file.write('''// ******************************************************************
// This file is automatically generated by generate_debug_settings.py
// ******************************************************************

#include "DebugSettings.h"


DebugSettingsStruct debugSettings;


void DebugSettings_Init()
{
''')


for row in debugsettings:
    file.write("    debugSettings." + row[0] + " = " + str(row[4]) + ";\n")


file.write("}\n\n")
file.close()

#************ generate DebugMenu.c ************
file = open(destination_folder + "DebugMenu.c", "w")

file.write('''// ******************************************************************
// This file is automatically generated by generate_debug_settings.py
// ******************************************************************

#include "..\DebugMenu.h"

#include <genesis.h>

#include "..\Debug.h"
#include "DebugSettings.h"
#include "..\JoypadManager.h"
#include "..\LevelManager.h"
#include "..\MathUtils.h"
#include "..\StateManager.h"
#include "..\VDP_Extra.h"
#include "AllLevels.h"

#include "..\exported\ExportedGenResSprites.h"


#define MENU_ITEM_RETURN 0
''')

count = 1
for row in debugsettings:
    file.write('#define MENU_ITEM_' + row[0].upper() + " " + str(count) + "\n")
    count += 1

file.write('#define MAX_MENU_ITEMS ' + str(count))
        

file.write('''

u16 menuItem = 0;
u16* currentValue;
u16 currentValueMin;
u16 currentValueMax;

#define X_TILE_POSITION 2
#define SETTING_XPOSITION 27
#define FIRST_TEXT_ROW 10
u16 yPositionCounter;

void StartDrawingText()
{
    yPositionCounter = 0;
}

void DrawTextString(const char* string)
{
    VDP_drawText(string, X_TILE_POSITION, FIRST_TEXT_ROW + yPositionCounter);
    yPositionCounter++;
}

void DrawArrows()
{
    if (yPositionCounter == menuItem)
    {
        VDP_drawText("<", SETTING_XPOSITION - 1, FIRST_TEXT_ROW + yPositionCounter);
        VDP_drawText(">", SETTING_XPOSITION + 10, FIRST_TEXT_ROW + yPositionCounter);
    }
    else
    {
        VDP_drawText(" ", SETTING_XPOSITION - 1, FIRST_TEXT_ROW + yPositionCounter);
        VDP_drawText(" ", SETTING_XPOSITION + 10, FIRST_TEXT_ROW + yPositionCounter);
    }
}

void DrawStringSetting(const char* description, u16 settingValue, const char* const stringArray[])
{
    VDP_drawText(description, X_TILE_POSITION, FIRST_TEXT_ROW + yPositionCounter);

    VDP_drawText(stringArray[settingValue], SETTING_XPOSITION, FIRST_TEXT_ROW + yPositionCounter);

    DrawArrows();

    yPositionCounter++;
}


void DrawIntSetting(const char* description, u16 settingValue)
{
    VDP_drawText(description, X_TILE_POSITION, FIRST_TEXT_ROW + yPositionCounter);

    char tempString[3];

    intToStr(settingValue, &tempString[0], 3);

    VDP_drawText(tempString, SETTING_XPOSITION, FIRST_TEXT_ROW + yPositionCounter);

    DrawArrows();

    yPositionCounter++;
}

void DrawFunctionSetting(const char* description)
{
    VDP_drawText(description, X_TILE_POSITION, FIRST_TEXT_ROW + yPositionCounter);

    yPositionCounter++;
}


void DrawBoolSetting(const char* description, u16 settingValue)
{
    VDP_drawText(description, X_TILE_POSITION, FIRST_TEXT_ROW + yPositionCounter);

    if (settingValue)
    {
        VDP_drawText("On ", SETTING_XPOSITION, FIRST_TEXT_ROW + yPositionCounter);
    }
    else
    {
        VDP_drawText("Off", SETTING_XPOSITION, FIRST_TEXT_ROW + yPositionCounter);
    }

    DrawArrows();

    yPositionCounter++;
}

void DebugMenu_Draw()
{
    StartDrawingText();
    DrawTextString("Return");
''')

for row in debugsettings:
    if row[1] == "bool":
        file.write('    DrawBoolSetting("' + row[5] + ':", debugSettings.' + row[0] + ');\n')
    if row[1] == "int":
        file.write('    DrawIntSetting("' + row[5] + ':", debugSettings.' + row[0] + ');\n')
    if row[1] == "stringarray":
        file.write('    DrawStringSetting("' + row[5] + ':", debugSettings.' + row[0] + ', ' + row[8] + ');\n')
    if row[1] == "function":
        file.write('    DrawFunctionSetting("' + row[5] + '");\n')

file.write('''}

void UpdateCurrentValue()
{
    switch (menuItem)
    {
''')

for row in debugsettings:
    file.write('    case MENU_ITEM_' + row[0].upper() + ":\n")
    file.write('        currentValue = &debugSettings.' + row[0] + ';\n')
    file.write('        currentValueMin = ' + str(row[2]) + ';\n')

    if row[1] == "stringarray":
        file.write('        currentValueMax = ' + str(row[9]) + '- 1;\n')
    else:
        file.write('        currentValueMax = ' + str(row[3]) + ';\n')

    file.write('        break;\n')
file.write('''    }
}

void DebugMenu_Update()
{
    // handle input
    if (joystickState.upPressed)
    {
        if (menuItem == 0)
        {
            menuItem = MAX_MENU_ITEMS - 1;
        }
        else
        {
            menuItem--;
        }

        UpdateCurrentValue();
        DebugMenu_Draw();
    }

    if (joystickState.downPressed)
    {
        if (menuItem == MAX_MENU_ITEMS - 1)
        {
            menuItem = 0;
        }
        else
        {
            menuItem++;
        }

        UpdateCurrentValue();
        DebugMenu_Draw();
    }

    if (joystickState.startPressed)
    {
        if (menuItem == MENU_ITEM_RETURN)
        {
            StateManager_ChangeToState(LevelManager_ReturnFromMenu);
            return;
        }
        else
        {\n''')

for row in debugsettings:
    if row[6] is not "NULL":
        index = debugsettings.index(row)
        file.write('            if (menuItem == MENU_ITEM_' + row[0].upper() + ')\n')
        file.write('            {\n')
        file.write('                 ' + row[6] + '(' + row[7] + ');\n')
        file.write('            }\n')

file.write('''        }
    }
    
    if (menuItem > 0)
    {
        if (joystickState.leftPressed)
        {
            if (*currentValue > currentValueMin)
            {
                (*currentValue)--;
            }
            else
            {
                *currentValue = currentValueMax;
            }

            DebugMenu_Draw();
        }

        if (joystickState.rightPressed)
        {
            if (*currentValue < currentValueMax)
            {
                (*currentValue)++;
            }
            else
            {
                *currentValue = currentValueMin;
            }

            DebugMenu_Draw();
        }
    }

    // draw cursor
    VDP_resetSprites();
    u16 tileAttr = TILE_ATTR_FULL(PAL0,0,0,0,1);
    VDP_setSprite(0, TILE_TO_PIXEL(X_TILE_POSITION - 1), TILE_TO_PIXEL(10) + TILE_TO_PIXEL(menuItem), SPRITE_SIZE(1,1), tileAttr, 0);
    MyVDP_waitVSync();
    VDP_updateSprites();
}

void DebugMenu_Start()
{
    MyVDP_waitVSync();

    VDP_setEnable(0);

    VDPManager_ResetDisplay();
    DebugMenu_Draw();

    VDP_setEnable(1);

    menuItem = 0;

    StateManager_ChangeToState(DebugMenu_Update);
}
''')

file.close()

